@@ -1,8 +1,8 @@
 /*
  *  Hamlib Alinco backend - main file
- *  Copyright (c) 2001-2004 by Stephane Fillod
+ *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: alinco.c,v 1.24 2004/09/26 08:34:53 fillods Exp $
+ *	$Id: alinco.c,v 1.27 2005/04/10 21:47:12 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -109,28 +109,33 @@ int alinco_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
 
 	retval = write_block(&rs->rigport, cmd, cmd_len);
 	if (retval != RIG_OK)
-			return retval;
+		return retval;
 
 	/*
 	 * Transceiver sends an echo of cmd followed by a CR/LF
 	 * TODO: check whether cmd and echobuf match (optional)
 	 */
 	retval = read_string(&rs->rigport, echobuf, BUFSZ, LF, strlen(LF));
-	/* if (retval <= 3)
-			return retval; */
+	if (retval < 0)
+		return retval;
 
 	/* no data expected, check for OK returned */
 	if (!data || !data_len) {
 	    	retval = read_string(&rs->rigport, echobuf, BUFSZ, LF, strlen(LF));
+		if (retval < 0)
+			return retval;
 		retval -= 2;
 		echobuf[retval] = 0;
 		if (strcmp(echobuf, "OK") == 0) 
 			return RIG_OK;
 		else
-			return RIG_ERJCTED;
+			return -RIG_ERJCTED;
 	}
 
-	*data_len = read_string(&rs->rigport, data, BUFSZ, LF, strlen(LF));
+	retval = read_string(&rs->rigport, data, BUFSZ, LF, strlen(LF));
+	if (retval < 0)
+		return retval;
+	*data_len = retval;
 
 	/* strip CR/LF from string
 	 */
@@ -214,7 +219,7 @@ int alinco_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 				return -RIG_EINVAL;
 
 		/* at least 6 digits */
-		freq_len = sprintf(freqbuf, AL CMD_RXFREQ "%06Ld" EOM, (long long)freq);
+		freq_len = sprintf(freqbuf, AL CMD_RXFREQ "%06"PRIll EOM, (long long)freq);
 
 		return alinco_transaction (rig, freqbuf, freq_len, NULL, NULL);
 }
@@ -254,7 +259,7 @@ int alinco_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
 		/* extract RX freq */
 		freqbuf[16] = '\0';
-		sscanf(freqbuf+6, "%"FREQFMT, freq);
+		sscanf(freqbuf+6, "%"SCNfreq, freq);
 
 		return RIG_OK;
 }
@@ -409,7 +414,7 @@ int alinco_set_split_freq(RIG *rig, vfo_t vfo, freq_t tx_freq)
 				return -RIG_EINVAL;
 
 		/* at least 6 digits */
-		freq_len = sprintf(freqbuf, AL CMD_TXFREQ "%06Ld" EOM, (long long)tx_freq);
+		freq_len = sprintf(freqbuf, AL CMD_TXFREQ "%06"PRIll EOM, (long long)tx_freq);
 
 		retval = alinco_transaction (rig, freqbuf, freq_len, NULL, NULL);
 
@@ -431,7 +436,7 @@ int alinco_get_split_freq(RIG *rig, vfo_t vfo, freq_t *tx_freq)
 
 		/* extract TX freq first, as RX kills freqbuf[16] */
 		freqbuf[26] = '\0';
-		sscanf(freqbuf+16, "%"FREQFMT, tx_freq);
+		sscanf(freqbuf+16, "%"SCNfreq, tx_freq);
 
 		return RIG_OK;
 }
