@@ -1,8 +1,8 @@
 /*
  *  Hamlib AOR backend - main header
- *  Copyright (c) 2000-2004 by Stephane Fillod
+ *  Copyright (c) 2000-2005 by Stephane Fillod
  *
- *	$Id: aor.h,v 1.17 2004/09/07 20:40:20 fillods Exp $
+ *	$Id: aor.h,v 1.22 2005/04/20 14:48:49 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -25,7 +25,18 @@
 
 #include <hamlib/rig.h>
 
-#define BACKEND_VER "0.2"
+#define BACKEND_VER "0.4"
+
+
+int format8k_mode(RIG *rig, char *buf, rmode_t mode, pbwidth_t width);
+int parse8k_aor_mode(RIG *rig, char aormode, char aorwidth, rmode_t *mode, pbwidth_t *width);
+
+struct aor_priv_caps {
+	int (*format_mode)(RIG *rig, char *buf, rmode_t mode, pbwidth_t width);
+	int (*parse_aor_mode)(RIG *rig, char aormode, char aorwidth, rmode_t *mode, pbwidth_t *width);
+	char bank_base1;
+	char bank_base2;
+};
 
 int aor_close(RIG *rig);
 
@@ -45,12 +56,21 @@ int aor_set_powerstat(RIG *rig, powerstat_t status);
 int aor_vfo_op(RIG *rig, vfo_t vfo, vfo_op_t op);
 const char *aor_get_info(RIG *rig);
 
+int aor_set_mem(RIG *rig, vfo_t vfo, int ch);
+int aor_get_mem(RIG *rig, vfo_t vfo, int *ch);
+int aor_set_bank(RIG *rig, vfo_t vfo, int bank);
+
+int aor_get_channel(RIG *rig, channel_t *chan);
+int aor_set_channel(RIG *rig, const channel_t *chan);
+int aor_get_chan_all_cb (RIG * rig, chan_cb_t chan_cb, rig_ptr_t);
+
+extern const struct rig_caps ar2700_caps;
 extern const struct rig_caps ar8200_caps;
 extern const struct rig_caps ar8000_caps;
+extern const struct rig_caps ar8600_caps;
 extern const struct rig_caps ar5000_caps;
 extern const struct rig_caps ar3000a_caps;
 extern const struct rig_caps ar7030_caps;
 extern const struct rig_caps ar3030_caps;
 
 #endif /* _AOR_H */
-
