@@ -1,8 +1,8 @@
 /*
  *  Hamlib AOR backend - AR3000 description
- *  Copyright (c) 2000-2004 by Stephane Fillod
+ *  Copyright (c) 2000-2005 by Stephane Fillod
  *
- *	$Id: ar3000.c,v 1.5 2004/09/07 20:37:41 fillods Exp $
+ *	$Id: ar3000.c,v 1.9 2005/04/10 21:47:12 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -74,9 +74,9 @@ const struct rig_caps ar3000a_caps = {
 .rig_model =  RIG_MODEL_AR3000A,
 .model_name = "AR3000A",
 .mfg_name =  "AOR",
-.version =  "0.2",
+.version =  "0.4",
 .copyright =  "LGPL",
-.status =  RIG_STATUS_UNTESTED,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_SCANNER,
 .ptt_type =  RIG_PTT_NONE,
 .dcd_type =  RIG_DCD_NONE,
@@ -205,7 +205,13 @@ static int ar3k_transaction(RIG *rig, const char *cmd, int cmd_len, char *data,
 	if (!data || !data_len)
 		return RIG_OK;
 
-	*data_len = read_string(&rs->rigport, data, BUFSZ, EOM, strlen(EOM));
+	retval = read_string(&rs->rigport, data, BUFSZ, EOM, strlen(EOM));
+	if (retval == -RIG_ETIMEOUT)
+		retval = 0;
+	if (retval < 0)
+		return retval;
+
+	*data_len = retval;
 
 	return RIG_OK;
 }
@@ -264,7 +270,7 @@ int ar3k_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 	rfp = strchr(freqbuf, 'Y');
 	if (!rfp)
 		return -RIG_EPROTO;
-	sscanf(rfp+1,"%"FREQFMT, freq);
+	sscanf(rfp+1,"%"SCNfreq, freq);
 	*freq *= 10;
 
 	return RIG_OK;
@@ -319,7 +325,7 @@ int ar3k_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 	rfp = strchr(buf, 'Y');
 	if (!rfp)
 		return -RIG_EPROTO;
-	rfp += 10;
+	rfp += 11;
 
 	switch (*rfp) {
 	case 'Q':	*mode = RIG_MODE_FM; break;
