@@ -1,8 +1,8 @@
 /*
  *  Hamlib Drake backend - R-8B description
- *  Copyright (c) 2001-2003 by Stephane Fillod
+ *  Copyright (c) 2001-2004 by Stephane Fillod
  *
- *	$Id: r8b.c,v 1.4 2003/10/01 19:31:55 fillods Exp $
+ *	$Id: r8b.c,v 1.11 2005/04/10 21:57:13 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -30,18 +30,51 @@
 #include "drake.h"
 
 
-#define R8B_MODES (RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM|RIG_MODE_FM)
+#define R8B_MODES (RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM|RIG_MODE_ECSS|RIG_MODE_FM)
 
-#define R8B_FUNC (RIG_FUNC_NONE)
+#define R8B_FUNC (RIG_FUNC_MN|RIG_FUNC_LOCK|RIG_FUNC_NB)
 
-#define R8B_LEVEL_ALL (RIG_LEVEL_NONE)
+#define R8B_LEVEL_ALL (RIG_LEVEL_PREAMP|RIG_LEVEL_ATT|RIG_LEVEL_AGC|RIG_LEVEL_RAWSTR|RIG_LEVEL_STRENGTH)
 
-#define R8B_PARM_ALL (RIG_PARM_NONE)
+#define R8B_PARM_ALL (RIG_PARM_TIME)
 
-#define R8B_VFO (RIG_VFO_A|RIG_VFO_B)
+#define R8B_VFO (RIG_VFO_A|RIG_VFO_B|RIG_VFO_VFO|RIG_VFO_MEM)
 
-#define R8B_VFO_OPS (RIG_OP_UP|RIG_OP_DOWN|RIG_OP_CPY)
+#define R8B_VFO_OPS (RIG_OP_UP|RIG_OP_DOWN|RIG_OP_TO_VFO|RIG_OP_FROM_VFO|RIG_OP_MCL|RIG_OP_CPY)
 
+#define R8B_ANTS (RIG_ANT_1|RIG_ANT_2|RIG_ANT_3)
+
+#define R8B_STR_CAL { 16, { \
+		{   0, -60 }, \
+		{   1, -48 }, \
+		{  11, -42 }, \
+		{  27, -36 }, \
+		{  39, -30 }, \
+		{  51, -24 }, \
+		{  64, -18 }, \
+		{  80, -12 }, \
+		{  97, -6 }, \
+		{ 116,  0 }, \
+		{ 138, 10 }, \
+		{ 163, 20 }, \
+		{ 195, 30 }, \
+		{ 217, 40 }, \
+		{ 228, 50 }, \
+		{ 255, 60 }, \
+	} }
+
+/*
+ * channel caps.
+ */
+#define DRAKE_MEM_CAP {	\
+	.freq = 1,	\
+	.mode = 1,	\
+	.width = 1,	\
+	.ant = 1,	\
+	.funcs = 1,	\
+	.levels = RIG_LEVEL_AGC|RIG_LEVEL_ATT|RIG_LEVEL_PREAMP,	\
+	.channel_desc = 1,	\
+} 
 
 /*
  * R-8B rig capabilities.
@@ -54,9 +87,9 @@ const struct rig_caps r8b_caps = {
 .rig_model =  RIG_MODEL_DKR8B,
 .model_name = "R-8B",
 .mfg_name =  "Drake",
-.version =  "0.1",
+.version =  BACKEND_VER,
 .copyright =  "LGPL",
-.status =  RIG_STATUS_UNTESTED,		/* and only basic support */
+.status =  RIG_STATUS_UNTESTED,
 .rig_type =  RIG_TYPE_RECEIVER,
 .ptt_type =  RIG_PTT_NONE,
 .dcd_type =  RIG_DCD_NONE,
@@ -94,16 +127,17 @@ const struct rig_caps r8b_caps = {
 .vfo_ops =  R8B_VFO_OPS,
 
 .chan_list =  {
-		RIG_CHAN_END,	/* FIXME */
+		{   0,  999, RIG_MTYPE_MEM, DRAKE_MEM_CAP },
+		RIG_CHAN_END
 	},
 
-.rx_range_list1 =  { 
-	{kHz(10),MHz(30),R8B_MODES,-1,-1,R8B_VFO},
+.rx_range_list1 =  {
+	{kHz(10),MHz(30),R8B_MODES,-1,-1,R8B_VFO,R8B_ANTS},
 	RIG_FRNG_END,
   },
 .tx_range_list1 =  { RIG_FRNG_END, },
 .rx_range_list2 =  {
-	{kHz(10),MHz(30),R8B_MODES,-1,-1,R8B_VFO},
+	{kHz(10),MHz(30),R8B_MODES,-1,-1,R8B_VFO,R8B_ANTS},
 	RIG_FRNG_END,
   },
 .tx_range_list2 =  { RIG_FRNG_END, },
@@ -117,20 +151,41 @@ const struct rig_caps r8b_caps = {
 	},
         /* mode/filter list, remember: order matters! */
 .filters =  {
-		{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM, kHz(2.3)},	/* normal */
-		{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM, kHz(6)},	/* wide */
-		{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM, kHz(4)},
-		{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM, kHz(1.8)},	/* narrow */
-		{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_RTTY|RIG_MODE_AM, Hz(500)},
 		{RIG_MODE_FM, kHz(12)},
+		{RIG_MODE_AM|RIG_MODE_ECSS, kHz(6)},
+		{RIG_MODE_AM|RIG_MODE_ECSS, kHz(4)},
+		{RIG_MODE_SSB|RIG_MODE_RTTY, kHz(2.3)},
+		{RIG_MODE_SSB|RIG_MODE_RTTY, kHz(1.8)},
+		{RIG_MODE_SSB|RIG_MODE_RTTY, kHz(4)},
+		{RIG_MODE_CW, Hz(500)},
+		{RIG_MODE_CW, kHz(1.8)},
 		RIG_FLT_END,
 	},
+.str_cal = R8B_STR_CAL,
 .priv =  NULL,
 
-.set_freq =  drake_set_freq,
-.set_vfo =  drake_set_vfo,
-.set_mode =  drake_set_mode,
+.rig_init = drake_init,
+.rig_cleanup = drake_cleanup,
 
+.set_freq =  drake_set_freq,
+.get_freq =  drake_get_freq,
+.set_vfo =  drake_set_vfo,
+.get_vfo =  drake_get_vfo,
+.set_mode =  drake_set_mode,
+.get_mode =  drake_get_mode,
+.set_func = drake_set_func,
+.get_func = drake_get_func,
+.set_level = drake_set_level,
+.get_level = drake_get_level,
+.set_ant = drake_set_ant,
+.get_ant = drake_get_ant,
+.set_mem = drake_set_mem,
+.get_mem = drake_get_mem,
+.set_channel = drake_set_chan,
+.get_channel = drake_get_chan,
+.vfo_op = drake_vfo_op,
+.set_powerstat = drake_set_powerstat,
+.get_powerstat = drake_get_powerstat,
 .get_info =  drake_get_info,
 
 };
