@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - API header
- *  Copyright (c) 2000-2004 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2005 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rig.h,v 1.100 2004/09/25 14:31:38 fillods Exp $
+ *	$Id: rig.h,v 1.110 2006/01/15 14:23:16 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -23,6 +23,9 @@
 #ifndef _RIG_H
 #define _RIG_H 1
 
+#include <stdio.h>
+#include <stdarg.h>
+
 #include <hamlib/riglist.h>	/* list in another file to not mess up w/ this one */
 
 /*! \file rig.h
@@ -156,7 +159,8 @@ typedef enum rig_port_e {
   RIG_PORT_DTMF,		/*!< DTMF protocol bridge via another rig, eg. Kenwood Sky Cmd System */
   RIG_PORT_ULTRA,		/*!< IrDA Ultra protocol! */
   RIG_PORT_RPC,			/*!< RPC wrapper */
-  RIG_PORT_PARALLEL		/*!< Parallel port */
+  RIG_PORT_PARALLEL,		/*!< Parallel port */
+  RIG_PORT_USB			/*!< USB port */
 } rig_port_t;
 
 /**
@@ -252,7 +256,11 @@ typedef enum {
  * Frequency type unit in Hz, able to hold SHF frequencies.
  */
 typedef double freq_t;
-#define FREQFMT "lf"
+/** \brief printf(3) format to be used for freq_t type */
+#define PRIfreq "f"
+/** \brief scanf(3) format to be used for freq_t type */
+#define SCNfreq "lf"
+#define FREQFMT SCNfreq
 
 /**
  * \brief Short frequency type
@@ -326,10 +334,15 @@ typedef int vfo_t;
 #define RIG_VFO_C RIG_VFO_N(2)
 
 
-#define RIG_TARGETABLE_NONE 0x00
-#define RIG_TARGETABLE_FREQ 0x01
-#define RIG_TARGETABLE_MODE 0x02
-#define RIG_TARGETABLE_ALL  0xffffffffU
+/*
+ * targetable bitfields, for internal use.
+ * RIG_TARGETABLE_PURE means a pure targetable radio on every command
+ */
+#define RIG_TARGETABLE_NONE 0
+#define RIG_TARGETABLE_FREQ (1<<0)
+#define RIG_TARGETABLE_MODE (1<<1)
+#define RIG_TARGETABLE_PURE (1<<2)
+#define RIG_TARGETABLE_ALL  0x7fffffff
 
 
 #define RIG_PASSBAND_NORMAL s_Hz(0)
@@ -579,7 +592,7 @@ enum rig_level_e {
 		/*!< These ones are not settable */
 	RIG_LEVEL_RAWSTR =	(1<<26),/*!< Raw (A/D) value for signal strength, specific to each rig, arg int */
 	RIG_LEVEL_SQLSTAT =	(1<<27),/*!< SQL status, arg int (open=1/closed=0). Deprecated, use get_dcd instead */
-	RIG_LEVEL_SWR =		(1<<28),/*!< SWR, arg float */
+	RIG_LEVEL_SWR =		(1<<28),/*!< SWR, arg float [0.0..infinite] */
 	RIG_LEVEL_ALC =		(1<<29),/*!< ALC, arg float */
 	RIG_LEVEL_STRENGTH =	(1<<30) /*!< Effective (calibrated) signal strength relative to S9, arg int (dB) */
 	/*RIG_LEVEL_BWC =		(1<<31)*/ /*!< Bandwidth Control, arg int (Hz) */
@@ -992,6 +1005,9 @@ typedef struct cal_table cal_table_t;
 #define EMPTY_STR_CAL { 0, { { 0, 0 }, } }
 
 
+typedef int (*chan_cb_t) (RIG *, channel_t**, int, const chan_t*, rig_ptr_t);
+typedef int (*confval_cb_t) (RIG *, const struct confparams *, value_t *, rig_ptr_t);
+
 /**
  * \brief Rig data structure.
  *
@@ -1205,6 +1221,14 @@ struct rig_caps {
 
   const char *(*get_info) (RIG * rig);
 
+  int (*set_chan_all_cb) (RIG * rig, chan_cb_t chan_cb, rig_ptr_t);
+  int (*get_chan_all_cb) (RIG * rig, chan_cb_t chan_cb, rig_ptr_t);
+
+  int (*set_mem_all_cb) (RIG * rig, chan_cb_t chan_cb, confval_cb_t parm_cb, rig_ptr_t);
+  int (*get_mem_all_cb) (RIG * rig, chan_cb_t chan_cb, confval_cb_t parm_cb, rig_ptr_t);
+
+  const char *clone_combo_set;	/*<! String describing key combination to enter load cloning mode */
+  const char *clone_combo_get;	/*<! String describing key combination to enter save cloning mode */
 };
 
 /**
@@ -1219,9 +1243,7 @@ typedef struct {
 	dcd_type_t dcd;		/*!< DCD port type */
   } type;
   int fd;			/*!< File descriptor */
-#if defined(__CYGWIN__) || defined(_WIN32)
-  void* deprecated_handle;	/*!< Place holder for deprecated field */
-#endif
+  void* handle;			/*!< handle for USB */
 
   int write_delay;		/*!< Delay between each byte sent out, in ms */
   int post_write_delay;		/*!< Delay between each commands send out, in ms */
@@ -1243,8 +1265,19 @@ typedef struct {
 	struct {
 		int pin;	/*!< Parrallel port pin number */
 	} parallel;		/*!< parallel attributes */
+	struct {
+		int vid;	/*!< Vendor ID */
+		int pid;	/*!< Product ID */
+		int conf;	/*!< Configuration */
+		int iface;	/*!< interface */
+		int alt;	/*!< alternate */
+	} usb;			/*!< USB attributes */
   } parm;			/*!< Port parameter union */
-} port_t;
+} hamlib_port_t;
+
+#if !defined(__APPLE__) || !defined(__cplusplus)
+typedef hamlib_port_t port_t;
+#endif
 
 
 /** 
@@ -1260,9 +1293,9 @@ struct rig_state {
 	/*
 	 * overridable fields
 	 */
-  port_t rigport;	/*!< Rig port (internal use). */
-  port_t pttport;	/*!< PTT port (internal use). */
-  port_t dcdport;	/*!< DCD port (internal use). */
+  hamlib_port_t rigport;	/*!< Rig port (internal use). */
+  hamlib_port_t pttport;	/*!< PTT port (internal use). */
+  hamlib_port_t dcdport;	/*!< DCD port (internal use). */
 
   double vfo_comp;	/*!< VFO compensation in PPM, 0.0 to disable */
 
@@ -1318,6 +1351,8 @@ struct rig_state {
 };
 
 
+typedef int (*vprintf_cb_t) (enum rig_debug_level_e, rig_ptr_t, const char *, va_list);
+
 typedef int (*freq_cb_t) (RIG *, vfo_t, freq_t, rig_ptr_t);
 typedef int (*mode_cb_t) (RIG *, vfo_t, rmode_t, pbwidth_t, rig_ptr_t);
 typedef int (*vfo_cb_t) (RIG *, vfo_t, rig_ptr_t);
@@ -1503,6 +1538,16 @@ extern HAMLIB_EXPORT(scan_t) rig_has_scan HAMLIB_PARAMS((RIG *rig, scan_t scan))
 extern HAMLIB_EXPORT(int) rig_set_channel HAMLIB_PARAMS((RIG *rig, const channel_t *chan));	/* mem */
 extern HAMLIB_EXPORT(int) rig_get_channel HAMLIB_PARAMS((RIG *rig, channel_t *chan));
 
+extern HAMLIB_EXPORT(int) rig_set_chan_all HAMLIB_PARAMS((RIG *rig, const channel_t chans[]));
+extern HAMLIB_EXPORT(int) rig_get_chan_all HAMLIB_PARAMS((RIG *rig, channel_t chans[]));
+extern HAMLIB_EXPORT(int) rig_set_chan_all_cb HAMLIB_PARAMS((RIG *rig, chan_cb_t chan_cb, rig_ptr_t));
+extern HAMLIB_EXPORT(int) rig_get_chan_all_cb HAMLIB_PARAMS((RIG *rig, chan_cb_t chan_cb, rig_ptr_t));
+
+extern HAMLIB_EXPORT(int) rig_set_mem_all_cb HAMLIB_PARAMS((RIG *rig, chan_cb_t chan_cb, confval_cb_t parm_cb, rig_ptr_t));
+extern HAMLIB_EXPORT(int) rig_get_mem_all_cb HAMLIB_PARAMS((RIG *rig, chan_cb_t chan_cb, confval_cb_t parm_cb, rig_ptr_t));
+extern HAMLIB_EXPORT(int) rig_set_mem_all HAMLIB_PARAMS((RIG *rig, const channel_t *chan, const struct confparams *, const value_t *));
+extern HAMLIB_EXPORT(int) rig_get_mem_all HAMLIB_PARAMS((RIG *rig, channel_t *chan, const struct confparams *, value_t *));
+
 extern HAMLIB_EXPORT(int) rig_set_trn HAMLIB_PARAMS((RIG *rig, int trn));
 extern HAMLIB_EXPORT(int) rig_get_trn HAMLIB_PARAMS((RIG *rig, int *trn));
 extern HAMLIB_EXPORT(int) rig_set_freq_callback HAMLIB_PARAMS((RIG *, freq_cb_t, rig_ptr_t));
@@ -1524,26 +1569,29 @@ extern HAMLIB_EXPORT(pbwidth_t) rig_passband_wide HAMLIB_PARAMS((RIG *rig, rmode
 extern HAMLIB_EXPORT(const char *) rigerror HAMLIB_PARAMS((int errnum));
 
 extern HAMLIB_EXPORT(int) rig_setting2idx HAMLIB_PARAMS((setting_t s));
-#define rig_idx2setting(i) (1ULL<<(i))
+#define rig_idx2setting(i) (1UL<<(i))
 
 /*
  * Even if these functions are prefixed with "rig_", they are not rig specific
  * Maybe "hamlib_" would have been better. Let me know. --SF
  */
 extern HAMLIB_EXPORT(void) rig_set_debug HAMLIB_PARAMS((enum rig_debug_level_e debug_level));
+#define rig_set_debug_level(level) rig_set_debug(level)
 extern HAMLIB_EXPORT(int) rig_need_debug HAMLIB_PARAMS((enum rig_debug_level_e debug_level));
 extern HAMLIB_EXPORT(void) rig_debug HAMLIB_PARAMS((enum rig_debug_level_e debug_level, const char *fmt, ...));
+extern HAMLIB_EXPORT(vprintf_cb_t) rig_set_debug_callback HAMLIB_PARAMS((vprintf_cb_t cb, rig_ptr_t arg));
+extern HAMLIB_EXPORT(FILE*) rig_set_debug_file HAMLIB_PARAMS((FILE *stream));
 
 extern HAMLIB_EXPORT(int) rig_register HAMLIB_PARAMS((const struct rig_caps *caps));
 extern HAMLIB_EXPORT(int) rig_unregister HAMLIB_PARAMS((rig_model_t rig_model));
 extern HAMLIB_EXPORT(int) rig_list_foreach HAMLIB_PARAMS((int (*cfunc)(const struct rig_caps*, rig_ptr_t), rig_ptr_t data));
 extern HAMLIB_EXPORT(int) rig_load_backend HAMLIB_PARAMS((const char *be_name));
 extern HAMLIB_EXPORT(int) rig_check_backend HAMLIB_PARAMS((rig_model_t rig_model));
-extern HAMLIB_EXPORT(int) rig_load_all_backends HAMLIB_PARAMS(());
+extern HAMLIB_EXPORT(int) rig_load_all_backends HAMLIB_PARAMS((void));
 
-typedef int (*rig_probe_func_t)(const port_t *, rig_model_t, rig_ptr_t);
-extern HAMLIB_EXPORT(int) rig_probe_all HAMLIB_PARAMS((port_t *p, rig_probe_func_t, rig_ptr_t));
-extern HAMLIB_EXPORT(rig_model_t) rig_probe HAMLIB_PARAMS((port_t *p));
+typedef int (*rig_probe_func_t)(const hamlib_port_t *, rig_model_t, rig_ptr_t);
+extern HAMLIB_EXPORT(int) rig_probe_all HAMLIB_PARAMS((hamlib_port_t *p, rig_probe_func_t, rig_ptr_t));
+extern HAMLIB_EXPORT(rig_model_t) rig_probe HAMLIB_PARAMS((hamlib_port_t *p));
 
 
 /* Misc calls */
