@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - list of known rigs
- *  Copyright (c) 2000-2004 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2005 by Stephane Fillod and Frank Singleton
  *
- *	$Id: riglist.h,v 1.49 2004/11/15 16:54:43 fillods Exp $
+ *	$Id: riglist.h,v 1.58 2005/11/17 22:17:38 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -63,7 +63,7 @@
 #define RIG_MODEL_FT847 RIG_MAKE_MODEL(RIG_YAESU, 1)
 #define RIG_MODEL_FT1000 RIG_MAKE_MODEL(RIG_YAESU, 2)
 #define RIG_MODEL_FT1000D RIG_MAKE_MODEL(RIG_YAESU, 3)
-#define RIG_MODEL_FT1000MP RIG_MAKE_MODEL(RIG_YAESU, 4)
+#define RIG_MODEL_FT1000MPMKV RIG_MAKE_MODEL(RIG_YAESU, 4)
 #define RIG_MODEL_FT747 RIG_MAKE_MODEL(RIG_YAESU, 5)
 #define RIG_MODEL_FT757 RIG_MAKE_MODEL(RIG_YAESU, 6)
 #define RIG_MODEL_FT757GXII RIG_MAKE_MODEL(RIG_YAESU, 7)
@@ -83,6 +83,9 @@
 #define RIG_MODEL_FT100 RIG_MAKE_MODEL(RIG_YAESU, 21)
 #define RIG_MODEL_FT857 RIG_MAKE_MODEL(RIG_YAESU, 22)
 #define RIG_MODEL_FT897 RIG_MAKE_MODEL(RIG_YAESU, 23)
+#define RIG_MODEL_FT1000MP RIG_MAKE_MODEL(RIG_YAESU, 24)
+#define RIG_MODEL_FT1000MPMKVFLD RIG_MAKE_MODEL(RIG_YAESU, 25)
+#define RIG_MODEL_VR5000 RIG_MAKE_MODEL(RIG_YAESU, 26)
 
 	/*
 	 * Kenwood
@@ -116,6 +119,7 @@
 #define RIG_MODEL_TS140S RIG_MAKE_MODEL(RIG_KENWOOD, 25)
 #define RIG_MODEL_TMD700 RIG_MAKE_MODEL(RIG_KENWOOD, 26)
 #define RIG_MODEL_TMV7 RIG_MAKE_MODEL(RIG_KENWOOD, 27)
+#define RIG_MODEL_TS480 RIG_MAKE_MODEL(RIG_KENWOOD, 28)
 
 	/*
 	 * Icom
@@ -174,6 +178,8 @@
 #define RIG_MODEL_IC7800 RIG_MAKE_MODEL(RIG_ICOM, 56)
 #define RIG_MODEL_IC756PROIII RIG_MAKE_MODEL(RIG_ICOM, 57)
 #define RIG_MODEL_ICR20 RIG_MAKE_MODEL(RIG_ICOM, 58)
+#define RIG_MODEL_IC7000 RIG_MAKE_MODEL(RIG_ICOM, 60)
+/* next one is 61 */
 
 	/*
 	 * Optoelectronics (CI-V)
@@ -188,6 +194,7 @@
 	 */
 #define RIG_MODEL_OMNIVI RIG_MAKE_MODEL(RIG_ICOM, 50)
 #define RIG_MODEL_OMNIVIP RIG_MAKE_MODEL(RIG_ICOM, 51) /* OMNI-VI+ */
+#define RIG_MODEL_PARAGON2 RIG_MAKE_MODEL(RIG_ICOM, 59)
 
 	/*
 	 * Icom PCR
@@ -214,6 +221,7 @@
 #define RIG_MODEL_AR16 RIG_MAKE_MODEL(RIG_AOR, 10)
 #define RIG_MODEL_SDU5500 RIG_MAKE_MODEL(RIG_AOR, 11)
 #define RIG_MODEL_SDU5000 RIG_MAKE_MODEL(RIG_AOR, 12)
+#define RIG_MODEL_AR8600 RIG_MAKE_MODEL(RIG_AOR, 13)
 
 	/*
 	 * JRC
@@ -278,6 +286,7 @@
 #define RIG_MODEL_RA3790 RIG_MAKE_MODEL(RIG_RACAL, 1)
 #define RIG_MODEL_RA3720 RIG_MAKE_MODEL(RIG_RACAL, 2)
 #define RIG_MODEL_RA6790 RIG_MAKE_MODEL(RIG_RACAL, 3)
+#define RIG_MODEL_RA3710 RIG_MAKE_MODEL(RIG_RACAL, 4)
 
 	/*
 	 * Watkins-Johnson
@@ -336,6 +345,7 @@
 #define RIG_MODEL_TT526 RIG_MAKE_MODEL(RIG_TENTEC, 6)	/* 6N2 */
 #define RIG_MODEL_TT516 RIG_MAKE_MODEL(RIG_TENTEC, 7)	/* Argonaut V */
 #define RIG_MODEL_TT565 RIG_MAKE_MODEL(RIG_TENTEC, 8)	/* Orion */
+#define RIG_MODEL_TT585 RIG_MAKE_MODEL(RIG_TENTEC, 9)	/* Paragon */
 
 	/*
 	 * Alinco
@@ -372,7 +382,7 @@
 #define RIG_MODEL_MC4020 RIG_MAKE_MODEL(RIG_GNURADIO, 2)	/* MC4020 */
 #define RIG_MODEL_GRAUDIO RIG_MAKE_MODEL(RIG_GNURADIO, 3)	/* Sound card source */
 #define RIG_MODEL_GRAUDIOIQ RIG_MAKE_MODEL(RIG_GNURADIO, 4)	/* I&Q stereo sound card source */
-#define RIG_MODEL_USRP RIG_MAKE_MODEL(RIG_GNURADIO, 5)	/* Universal Software Radio Peripheral */
+#define RIG_MODEL_USRP_G RIG_MAKE_MODEL(RIG_GNURADIO, 5)	/* Universal Software Radio Peripheral */
 
 	/*
 	 * Microtune tuners
@@ -381,6 +391,7 @@
 #define RIG_BACKEND_MICROTUNE "microtune"
 #define RIG_MODEL_MICROTUNE_4937 RIG_MAKE_MODEL(RIG_MICROTUNE, 1)	/* eval board */
 #define RIG_MODEL_MICROTUNE_4702 RIG_MAKE_MODEL(RIG_MICROTUNE, 2)	/* Alan's */
+#define RIG_MODEL_MICROTUNE_4707 RIG_MAKE_MODEL(RIG_MICROTUNE, 3)
 
 	/*
 	 * TAPR
@@ -395,6 +406,7 @@
 #define RIG_FLEXRADIO 23
 #define RIG_BACKEND_FLEXRADIO "flexradio"
 #define RIG_MODEL_SDR1000 RIG_MAKE_MODEL(RIG_FLEXRADIO, 1)
+#define RIG_MODEL_SDR1000RFE RIG_MAKE_MODEL(RIG_FLEXRADIO, 2)
 
 
 	/*
@@ -411,6 +423,9 @@
 #define RIG_BACKEND_KIT "kit"
 #define RIG_MODEL_ELEKTOR304 RIG_MAKE_MODEL(RIG_KIT, 1)
 #define RIG_MODEL_DRT1 RIG_MAKE_MODEL(RIG_KIT, 2)
+#define RIG_MODEL_DWT RIG_MAKE_MODEL(RIG_KIT, 3)
+#define RIG_MODEL_USRP0 RIG_MAKE_MODEL(RIG_KIT, 4)	/* prototype */
+#define RIG_MODEL_USRP RIG_MAKE_MODEL(RIG_KIT, 5)
 
 	/*
 	 * SW/FM/TV tuner cards supported by Video4Linux,*BSD, ..
@@ -449,13 +464,11 @@ typedef int rig_model_t;
 		{ RIG_PCR, RIG_BACKEND_PCR }, \
 		{ RIG_AOR, RIG_BACKEND_AOR }, \
 		{ RIG_JRC, RIG_BACKEND_JRC }, \
-		{ RIG_RADIOSHACK, RIG_BACKEND_RADIOSHACK }, \
 		{ RIG_UNIDEN, RIG_BACKEND_UNIDEN }, \
 		{ RIG_DRAKE, RIG_BACKEND_DRAKE }, \
 		{ RIG_LOWE, RIG_BACKEND_LOWE }, \
 		{ RIG_RACAL, RIG_BACKEND_RACAL }, \
 		{ RIG_WJ, RIG_BACKEND_WJ }, \
-		{ RIG_EK, RIG_BACKEND_EK }, \
 		{ RIG_SKANTI, RIG_BACKEND_SKANTI }, \
 		{ RIG_WINRADIO, RIG_BACKEND_WINRADIO }, \
 		{ RIG_TENTEC, RIG_BACKEND_TENTEC }, \
@@ -478,6 +491,9 @@ typedef int rig_model_t;
  *		const char *backend;
  * } rig_backend_list[] = RIG_LIST;
  *
+ * TODO:
+ *
+	{ RIG_RADIOSHACK, RIG_BACKEND_RADIOSHACK }, \
  */
 
 #endif /* _RIGLIST_H */
