@@ -1,8 +1,8 @@
 /*
  *  Hamlib PCR backend - main file
- *  Copyright (c) 2001-2003 by Stephane Fillod and Darren Hatcher
+ *  Copyright (c) 2001-2005 by Stephane Fillod and Darren Hatcher
  *
- *	$Id: pcr.c,v 1.18 2003/04/16 22:30:41 fillods Exp $
+ *	$Id: pcr.c,v 1.22 2005/04/10 21:47:13 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -95,7 +96,7 @@ const int pcr1_ctcss_list[] = {
  * Otherwise, you'll get a nice seg fault. You've been warned!
  * TODO: error case handling
  */
-int pcr_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
+static int pcr_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
 {
 	int retval;
 	struct rig_state *rs;
@@ -106,25 +107,32 @@ int pcr_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *dat
 
 	retval = write_block(&rs->rigport, cmd, cmd_len);
 	if (retval != RIG_OK)
-			return retval;
+		return retval;
 
 	/* eat the first ack */
 #ifdef WANT_READ_STRING
 	retval = read_string(&rs->rigport, data, 1, "\x0a", 1);
 	if (retval < 0)
-			return retval;
+		return retval;
 	if (retval != 1)
-			return -RIG_EPROTO;
+		return -RIG_EPROTO;
 #else
 	retval = read_block(&rs->rigport, data, 1);
+	if (retval < 0)
+		return retval;
 #endif
 
 	/* here is the real response */
 #ifdef WANT_READ_STRING
-	*data_len = read_string(&rs->rigport, data, *data_len, "\x0a", 1);
+	retval = read_string(&rs->rigport, data, *data_len, "\x0a", 1);
 #else
-	*data_len = read_block( &rs->rigport, data, *data_len );
+	retval = read_block( &rs->rigport, data, *data_len );
 #endif
+	if (retval == -RIG_ETIMEOUT)
+		retval = 0;
+	if (retval < 0)
+		return retval;
+	*data_len = retval;
 
 	return RIG_OK;
 }
@@ -268,7 +276,7 @@ int pcr_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 
 		priv = (struct pcr_priv_data *)rig->state.priv;
 
-		freq_len = sprintf(freqbuf,"K0%010Ld0%c0%c00" EOM, freq, 
+		freq_len = sprintf(freqbuf,"K0%010"PRIll"0%c0%c00" EOM, (long long)freq, 
 						priv->last_mode, priv->last_filter);
 
 		ack_len = 6;
@@ -327,8 +335,8 @@ int pcr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 			case RIG_MODE_WFM:	pcrmode = MD_WFM; pcrfilter = FLT_230kHz; break;
 			case RIG_MODE_FM:	pcrmode = MD_FM; pcrfilter = FLT_15kHz; break;
 			default:
-				rig_debug(RIG_DEBUG_ERR,"pcr_set_mode: unsupported mode %d\n",
-								mode);
+				rig_debug(RIG_DEBUG_ERR,"%s: unsupported mode %d\n",
+						__FUNCTION__, mode);
 				return -RIG_EINVAL;
 		}
 
@@ -338,18 +346,18 @@ int pcr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 				 */
 			case RIG_PASSBAND_NORMAL: break;
 
-			case kHz(2.8):	pcrfilter = FLT_2_8kHz; break;
-			case kHz(6):	pcrfilter = FLT_6kHz; break;
-			case kHz(15):	pcrfilter = FLT_15kHz; break;
-			case kHz(50):	pcrfilter = FLT_50kHz; break;
-			case kHz(230):	pcrfilter = FLT_230kHz; break;
+			case s_kHz(2.8):	pcrfilter = FLT_2_8kHz; break;
+			case s_kHz(6):		pcrfilter = FLT_6kHz; break;
+			case s_kHz(15):		pcrfilter = FLT_15kHz; break;
+			case s_kHz(50):		pcrfilter = FLT_50kHz; break;
+			case s_kHz(230):	pcrfilter = FLT_230kHz; break;
 			default:
-				rig_debug(RIG_DEBUG_ERR,"pcr_set_mode: unsupported "
-							"width %d\n", width);
+				rig_debug(RIG_DEBUG_ERR,"%s: unsupported "
+						"width %d\n", __FUNCTION__, width);
 				return -RIG_EINVAL;
 		}
 
-		mdbuf_len = sprintf(mdbuf,"K0%010Ld0%c0%c00" EOM, priv->last_freq, 
+		mdbuf_len = sprintf(mdbuf,"K0%010"PRIll"0%c0%c00" EOM, (long long)priv->last_freq, 
 						pcrmode, pcrfilter);
 
 		ack_len = 6;
@@ -358,8 +366,8 @@ int pcr_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 				return retval;
 
 		if (ack_len != 6 && ack_len != 4) {
-				rig_debug(RIG_DEBUG_ERR,"pcr_set_mode: ack NG, len=%d\n",
-								ack_len);
+				rig_debug(RIG_DEBUG_ERR,"%s: ack NG, len=%d\n",
+						__FUNCTION__, ack_len);
 				return -RIG_ERJCTED;
 		}
 
