@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - toolbox
- *  Copyright (c) 2000-2003 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2005 by Stephane Fillod
  *
- *	$Id: misc.c,v 1.31 2003/12/24 09:07:52 fillods Exp $
+ *	$Id: misc.c,v 1.38 2005/04/09 09:49:12 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -39,55 +39,6 @@
 #include "misc.h"
 
 
-
-static int rig_debug_level = RIG_DEBUG_TRACE;
-
-/*
- * Do a hex dump of the unsigned char array.
- */
-
-#define DUMP_HEX_WIDTH 16
-
-void dump_hex(const unsigned char ptr[], size_t size)
-{
-  int i;
-  char buf[DUMP_HEX_WIDTH+1];
-
-  if (!rig_need_debug(RIG_DEBUG_TRACE))
-		  return;
-
-  buf[DUMP_HEX_WIDTH] = '\0';
-
-  for(i=0; i<size; i++) {
-    if (i % DUMP_HEX_WIDTH == 0)
-      rig_debug(RIG_DEBUG_TRACE,"%.4x\t",i);
-
-    rig_debug(RIG_DEBUG_TRACE," %.2x", ptr[i]);
-
-	if (ptr[i] >= ' ' && ptr[i] < 0x7f)
-		buf[i%DUMP_HEX_WIDTH] = ptr[i];
-	else
-		buf[i%DUMP_HEX_WIDTH] = '.';
-
-    if (i % DUMP_HEX_WIDTH == DUMP_HEX_WIDTH-1)
-      rig_debug(RIG_DEBUG_TRACE,"\t%s\n",buf);
-  }
-
-  /* Add some spaces in order to align right ASCII dump column */
-  if ((i / DUMP_HEX_WIDTH) > 0) {
-    int j;
-    for (j = i % DUMP_HEX_WIDTH; j < DUMP_HEX_WIDTH; j++)
-      rig_debug(RIG_DEBUG_TRACE,"   ");
-  }
-
-  if (i % DUMP_HEX_WIDTH != DUMP_HEX_WIDTH-1) {
-  	buf[i % DUMP_HEX_WIDTH] = '\0';
-    rig_debug(RIG_DEBUG_TRACE,"\t%s\n",buf);
-  }
-
-} 
-
-
 /*
  * Convert a long long (eg. frequency in Hz) to 4-bit BCD digits, 
  * packed two digits per octet, in little-endian order.
@@ -97,7 +48,7 @@ void dump_hex(const unsigned char ptr[], size_t size)
  * Hope the compiler will do a good job optimizing it (esp. w/ the 64bit freq)
  */
 unsigned char *
-to_bcd(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
+HAMLIB_API to_bcd(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
 {
 	int i;
 	unsigned char a;
@@ -106,15 +57,15 @@ to_bcd(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
 	/* '450'/3-> 5,0;x,4 */
 
 	for (i=0; i < bcd_len/2; i++) {
-			a = freq%10;
-			freq /= 10;
-			a |= (freq%10)<<4;
-			freq /= 10;
-			bcd_data[i] = a;
+		a = freq%10;
+		freq /= 10;
+		a |= (freq%10)<<4;
+		freq /= 10;
+		bcd_data[i] = a;
 	}
 	if (bcd_len&1) {
-			bcd_data[i] &= 0xf0;
-			bcd_data[i] |= freq%10;	/* NB: high nibble is left uncleared */
+		bcd_data[i] &= 0xf0;
+		bcd_data[i] |= freq%10;	/* NB: high nibble is left uncleared */
 	}
 
 	return bcd_data;
@@ -126,19 +77,19 @@ to_bcd(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
  *
  * Hope the compiler will do a good job optimizing it (esp. w/ the 64bit freq)
  */
-unsigned long long from_bcd(const unsigned char bcd_data[], unsigned bcd_len)
+unsigned long long HAMLIB_API from_bcd(const unsigned char bcd_data[], unsigned bcd_len)
 {
 	int i;
 	freq_t f = 0;
 
 	if (bcd_len&1)
-			f = bcd_data[bcd_len/2] & 0x0f;
+		f = bcd_data[bcd_len/2] & 0x0f;
 
 	for (i=(bcd_len/2)-1; i >= 0; i--) {
-			f *= 10;
-			f += bcd_data[i]>>4;
-			f *= 10;
-			f += bcd_data[i] & 0x0f;
+		f *= 10;
+		f += bcd_data[i]>>4;
+		f *= 10;
+		f += bcd_data[i] & 0x0f;
 	}
 	
 	return f;
@@ -148,7 +99,7 @@ unsigned long long from_bcd(const unsigned char bcd_data[], unsigned bcd_len)
  * Same as to_bcd, but in Big Endian mode
  */
 unsigned char *
-to_bcd_be(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
+HAMLIB_API to_bcd_be(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
 {
 	int i;
 	unsigned char a;
@@ -157,16 +108,16 @@ to_bcd_be(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
 	/* '450'/3 -> 4,5;0,x */
 
 	if (bcd_len&1) {
-			bcd_data[bcd_len/2] &= 0x0f;
-			bcd_data[bcd_len/2] |= (freq%10)<<4;	/* NB: low nibble is left uncleared */
-			freq /= 10;
+		bcd_data[bcd_len/2] &= 0x0f;
+		bcd_data[bcd_len/2] |= (freq%10)<<4;	/* NB: low nibble is left uncleared */
+		freq /= 10;
 	}
 	for (i=(bcd_len/2)-1; i >= 0; i--) {
-			a = freq%10;
-			freq /= 10;
-			a |= (freq%10)<<4;
-			freq /= 10;
-			bcd_data[i] = a;
+		a = freq%10;
+		freq /= 10;
+		a |= (freq%10)<<4;
+		freq /= 10;
+		bcd_data[i] = a;
 	}
 
 	return bcd_data;
@@ -175,61 +126,25 @@ to_bcd_be(unsigned char bcd_data[], unsigned long long freq, unsigned bcd_len)
 /*
  * Same as from_bcd, but in Big Endian mode
  */
-unsigned long long from_bcd_be(const unsigned char bcd_data[], unsigned bcd_len)
+unsigned long long HAMLIB_API from_bcd_be(const unsigned char bcd_data[], unsigned bcd_len)
 {
 	int i;
 	freq_t f = 0;
 
 	for (i=0; i < bcd_len/2; i++) {
-			f *= 10;
-			f += bcd_data[i]>>4;
-			f *= 10;
-			f += bcd_data[i] & 0x0f;
+		f *= 10;
+		f += bcd_data[i]>>4;
+		f *= 10;
+		f += bcd_data[i] & 0x0f;
 	}
 	if (bcd_len&1) {
-			f *= 10;
-			f += bcd_data[bcd_len/2]>>4;
+		f *= 10;
+		f += bcd_data[bcd_len/2]>>4;
 	}
 
 	return f;
 }
 
-/*
- * rig_set_debug
- * Change the current debug level
- */
-void rig_set_debug(enum rig_debug_level_e debug_level)
-{
-		rig_debug_level = debug_level;
-}
-
-/*
- * rig_need_debug
- * Usefull for dump_hex, etc.
- */
-int rig_need_debug(enum rig_debug_level_e debug_level)
-{
-		return (debug_level <= rig_debug_level);
-}
-
-/*
- * rig_debug
- * Debugging messages are done through stderr
- * TODO: add syslog support if needed
- */
-void rig_debug(enum rig_debug_level_e debug_level, const char *fmt, ...)
-{
-		va_list ap;
-
-		if (debug_level <= rig_debug_level) {
-				va_start(ap, fmt);
-				/*
-				 * Who cares about return code?
-				 */
-				vfprintf (stderr, fmt, ap);
-				va_end(ap);
-		}
-}
 
 #ifndef llabs
 #define llabs(a) ((a)<0?-(a):(a))
@@ -240,29 +155,29 @@ void rig_debug(enum rig_debug_level_e debug_level, const char *fmt, ...)
  * pretty print frequencies
  * str must be long enough. max can be as long as 17 chars
  */
-int sprintf_freq(char *str, freq_t freq)
+int HAMLIB_API sprintf_freq(char *str, freq_t freq)
 {
-		double f;
-		char *hz;
+	double f;
+	char *hz;
 
-		if (llabs(freq) >= GHz(1)) {
-				hz = "GHz";
-				f = (double)freq/GHz(1);
-		} else if (llabs(freq) >= MHz(1)) {
-				hz = "MHz";
-				f = (double)freq/MHz(1);
-		} else if (llabs(freq) >= kHz(1)) {
-				hz = "kHz";
-				f = (double)freq/kHz(1);
-		} else {
-				hz = "Hz";
-				f = (double)freq;
-		}
+	if (llabs(freq) >= GHz(1)) {
+		hz = "GHz";
+		f = (double)freq/GHz(1);
+	} else if (llabs(freq) >= MHz(1)) {
+		hz = "MHz";
+		f = (double)freq/MHz(1);
+	} else if (llabs(freq) >= kHz(1)) {
+		hz = "kHz";
+		f = (double)freq/kHz(1);
+	} else {
+		hz = "Hz";
+		f = (double)freq;
+	}
 
-		return sprintf (str, "%g %s", f, hz);
+	return sprintf (str, "%g %s", f, hz);
 }
 
-const char *strptrshift(rptr_shift_t shift)
+const char * HAMLIB_API rig_strptrshift(rptr_shift_t shift)
 {
 	switch (shift) {
 	case RIG_RPT_SHIFT_MINUS: return "+";
@@ -273,21 +188,21 @@ const char *strptrshift(rptr_shift_t shift)
 	return NULL;
 }
 
-const char *strstatus(enum rig_status_e status)
+const char * HAMLIB_API rig_strstatus(enum rig_status_e status)
 {
 	switch (status) {
 	case RIG_STATUS_ALPHA:
-			return "Alpha";
+		return "Alpha";
 	case RIG_STATUS_UNTESTED:
-			return "Untested";
+		return "Untested";
 	case RIG_STATUS_BETA:
-			return "Beta";
+		return "Beta";
 	case RIG_STATUS_STABLE:
-			return "Stable";
+		return "Stable";
 	case RIG_STATUS_BUGGY:
-			return "Buggy";
+		return "Buggy";
 	case RIG_STATUS_NEW:
-			return "New";
+		return "New";
 	}
 	return "";
 }
@@ -310,11 +225,14 @@ static struct {
 	{ RIG_MODE_PKTLSB, "PKTLSB" },
 	{ RIG_MODE_PKTUSB, "PKTUSB" },
 	{ RIG_MODE_PKTFM, "PKTFM" },
+	{ RIG_MODE_ECSSUSB, "ECSSUSB" },
+	{ RIG_MODE_ECSSLSB, "ECSSLSB" },
+	{ RIG_MODE_FAX, "FAX" },
 	{ RIG_MODE_NONE, "" },
 };
 
 
-rmode_t parse_mode(const char *s)
+rmode_t HAMLIB_API rig_parse_mode(const char *s)
 {
 	int i;
 
@@ -324,7 +242,7 @@ rmode_t parse_mode(const char *s)
 	return RIG_MODE_NONE;
 }
 
-const char * strrmode(rmode_t mode)
+const char * HAMLIB_API rig_strrmode(rmode_t mode)
 {
 	int i;
 
@@ -354,7 +272,7 @@ static struct {
 		{ RIG_VFO_NONE, "" },
 };
 
-vfo_t parse_vfo(const char *s)
+vfo_t HAMLIB_API rig_parse_vfo(const char *s)
 {
 	int i;
 
@@ -364,7 +282,7 @@ vfo_t parse_vfo(const char *s)
 	return RIG_VFO_NONE;
 }
 
-const char *strvfo(vfo_t vfo)
+const char * HAMLIB_API rig_strvfo(vfo_t vfo)
 {
 	int i;
 
@@ -414,17 +332,18 @@ static struct {
 	{ RIG_FUNC_NONE, "" },
 };
 
-setting_t parse_func(const char *s)
+setting_t HAMLIB_API rig_parse_func(const char *s)
 {
 	int i;
 
 	for (i=0 ; func_str[i].str != ""; i++)
-			if (!strcmp(s, func_str[i].str))
-					return func_str[i].func;
+		if (!strcmp(s, func_str[i].str))
+			return func_str[i].func;
+
 	return RIG_FUNC_NONE;
 }
 
-const char *strfunc(setting_t func)
+const char * HAMLIB_API rig_strfunc(setting_t func)
 {
 	int i;
 
@@ -474,17 +393,18 @@ static struct {
 	{ RIG_LEVEL_NONE, "" },
 };
 
-setting_t parse_level(const char *s)
+setting_t HAMLIB_API rig_parse_level(const char *s)
 {
 	int i;
 
 	for (i=0 ; level_str[i].str != ""; i++)
-			if (!strcmp(s, level_str[i].str))
-					return level_str[i].level;
+		if (!strcmp(s, level_str[i].str))
+			return level_str[i].level;
+
 	return RIG_LEVEL_NONE;
 }
 
-const char *strlevel(setting_t level)
+const char * HAMLIB_API rig_strlevel(setting_t level)
 {
 	int i;
 
@@ -512,17 +432,19 @@ static struct {
 	{ RIG_PARM_NONE, "" },
 };
 
-setting_t parse_parm(const char *s)
+setting_t HAMLIB_API rig_parse_parm(const char *s)
 {
 	int i;
 
 	for (i=0 ; parm_str[i].str != ""; i++)
-			if (!strcmp(s, parm_str[i].str))
-					return parm_str[i].parm;
+		if (!strcmp(s, parm_str[i].str))
+			return parm_str[i].parm;
+
 	return RIG_PARM_NONE;
 }
 
-const char *strparm(setting_t parm)
+
+const char * HAMLIB_API rig_strparm(setting_t parm)
 {
 	int i;
 
@@ -557,17 +479,18 @@ static struct {
 	{ RIG_OP_NONE, "" },
 };
 
-vfo_op_t parse_vfo_op(const char *s)
+vfo_op_t HAMLIB_API rig_parse_vfo_op(const char *s)
 {
 	int i;
 
 	for (i=0 ; vfo_op_str[i].str != ""; i++)
-			if (!strcmp(s, vfo_op_str[i].str))
-					return vfo_op_str[i].vfo_op;
+		if (!strcmp(s, vfo_op_str[i].str))
+			return vfo_op_str[i].vfo_op;
+
 	return RIG_OP_NONE;
 }
 
-const char *strvfop(vfo_op_t op)
+const char * HAMLIB_API rig_strvfop(vfo_op_t op)
 {
 	int i;
 
@@ -597,7 +520,7 @@ static struct {
 	{ RIG_SCAN_NONE, "" },
 };
 
-scan_t parse_scan(const char *s)
+scan_t HAMLIB_API rig_parse_scan(const char *s)
 {
 	int i;
 
@@ -610,7 +533,7 @@ scan_t parse_scan(const char *s)
 	return RIG_SCAN_NONE;
 }
 
-const char *strscan(scan_t rscan)
+const char * HAMLIB_API rig_strscan(scan_t rscan)
 {
 	int i;
 
@@ -625,7 +548,7 @@ const char *strscan(scan_t rscan)
 }
 
 
-rptr_shift_t parse_rptr_shift(const char *s)
+rptr_shift_t HAMLIB_API rig_parse_rptr_shift(const char *s)
 {
 	if (strcmp(s, "+") == 0)
 		return RIG_RPT_SHIFT_PLUS;
