@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - parallel communication low-level support
- *  Copyright (c) 2000-2004 by Stephane Fillod
+ *  Copyright (c) 2000-2005 by Stephane Fillod
  *
- *	$Id: parallel.c,v 1.1 2004/10/02 20:37:24 fillods Exp $
+ *	$Id: parallel.c,v 1.5 2005/10/27 20:34:16 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -55,11 +55,15 @@
 #include "hamlib/rig.h"
 #include "parallel.h"
 
-
-
 #ifdef HAVE_LINUX_PPDEV_H
 #include <linux/ppdev.h>
 #include <linux/parport.h>
+#endif
+
+#ifdef HAVE_DEV_PPBUS_PPI_H
+#include <dev/ppbus/ppi.h>
+#include <dev/ppbus/ppbconf.h>
+#endif
 
 /* 
  * These control port bits are active low.
@@ -75,16 +79,13 @@
  */
 #define SP_ACTIVE_LOW_BITS	0x80
 
-#endif
-
-
 
 /*
  * TODO: to be called before exiting: atexit(parport_cleanup)
  * void parport_cleanup() { ioctl(fd, PPRELEASE); }
  */
 
-int par_open(port_t *port)
+int par_open(hamlib_port_t *port)
 {
 	int fd;
 	int mode;
@@ -105,6 +106,14 @@ int par_open(port_t *port)
 		return -RIG_EIO;
 	}
 
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+
+	fd = open(port->pathname, O_RDWR);
+	if (fd < 0) {
+		rig_debug(RIG_DEBUG_ERR, "Opening device \"%s\": %s\n", port->pathname, strerror(errno));
+		return -RIG_EIO;
+	}
+
 #elif defined(WIN32)
 	fd = (int)CreateFile(port->pathname, GENERIC_READ | GENERIC_WRITE,
 		0, NULL, OPEN_EXISTING, 0, NULL);
@@ -120,9 +129,10 @@ int par_open(port_t *port)
 	return fd;
 }
 
-int par_close(port_t *port)
+int par_close(hamlib_port_t *port)
 {
 #ifdef HAVE_LINUX_PPDEV_H
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
 #elif defined(WIN32)
 	CloseHandle((HANDLE)(port->fd));
 	return RIG_OK;
@@ -130,12 +140,16 @@ int par_close(port_t *port)
 	return close(port->fd);
 }
 
-int HAMLIB_API par_write_data(port_t *port, unsigned char data)
+int HAMLIB_API par_write_data(hamlib_port_t *port, unsigned char data)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	int status;
 	status = ioctl(port->fd, PPWDATA, &data);
 	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+	int status;
+	status = ioctl(port->fd, PPISDATA, &data);
+	return status == 0 ? RIG_OK : -RIG_EIO;
 #elif defined(WIN32)
 	unsigned int dummy;
 
@@ -147,12 +161,16 @@ int HAMLIB_API par_write_data(port_t *port, unsigned char data)
 	return -RIG_ENIMPL;
 }
 
-int HAMLIB_API par_read_data(port_t *port, unsigned char *data)
+int HAMLIB_API par_read_data(hamlib_port_t *port, unsigned char *data)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	int status;
 	status = ioctl(port->fd, PPRDATA, data);
 	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+	int status;
+	status = ioctl(port->fd, PPIGDATA, &data);
+	return status == 0 ? RIG_OK : -RIG_EIO;
 #elif defined(WIN32)
 	char ret;
 	unsigned int dummy;
@@ -168,13 +186,18 @@ int HAMLIB_API par_read_data(port_t *port, unsigned char *data)
 }
 
 
-int HAMLIB_API par_write_control(port_t *port, unsigned char control)
+int HAMLIB_API par_write_control(hamlib_port_t *port, unsigned char control)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	int status;
 	unsigned char ctrl = control ^ CP_ACTIVE_LOW_BITS;
 	status = ioctl(port->fd, PPWCONTROL, &ctrl);
 	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+	int status;
+	unsigned char ctrl = control ^ CP_ACTIVE_LOW_BITS;
+	status = ioctl(port->fd, PPISCTRL, &ctrl);
+	return status == 0 ? RIG_OK : -RIG_EIO;
 #elif defined(WIN32)
   unsigned char ctr = control;
   unsigned char dummyc;
@@ -201,7 +224,7 @@ int HAMLIB_API par_write_control(port_t *port, unsigned char control)
 	return -RIG_ENIMPL;
 }
 
-int HAMLIB_API par_read_control(port_t *port, unsigned char *control)
+int HAMLIB_API par_read_control(hamlib_port_t *port, unsigned char *control)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	int status;
@@ -209,6 +232,12 @@ int HAMLIB_API par_read_control(port_t *port, unsigned char *control)
 	status = ioctl(port->fd, PPRCONTROL, &ctrl);
 	*control = ctrl ^ CP_ACTIVE_LOW_BITS;
 	return status == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+	int status;
+	unsigned char ctrl;
+	status = ioctl(port->fd, PPIGCTRL, &ctrl);
+	*control = ctrl ^ CP_ACTIVE_LOW_BITS;
+	return status == 0 ? RIG_OK : -RIG_EIO;
 #elif defined(WIN32)
 	char ret;
 	unsigned int dummy;
@@ -224,7 +253,7 @@ int HAMLIB_API par_read_control(port_t *port, unsigned char *control)
 }
 
 
-int HAMLIB_API par_read_status(port_t *port, unsigned char *status)
+int HAMLIB_API par_read_status(hamlib_port_t *port, unsigned char *status)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	int ret;
@@ -232,6 +261,12 @@ int HAMLIB_API par_read_status(port_t *port, unsigned char *status)
 	ret = ioctl(port->fd, PPRSTATUS, &sta);
 	*status = sta ^ SP_ACTIVE_LOW_BITS;
 	return ret == 0 ? RIG_OK : -RIG_EIO;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
+	int ret;
+	unsigned char sta;
+	ret = ioctl(port->fd, PPIGSTATUS, &sta);
+	*status = sta ^ SP_ACTIVE_LOW_BITS;
+	return ret == 0 ? RIG_OK : -RIG_EIO;
 #elif defined(WIN32)
 	unsigned char ret;
 	unsigned int dummy;
@@ -247,7 +282,7 @@ int HAMLIB_API par_read_status(port_t *port, unsigned char *status)
 }
 
 
-int HAMLIB_API par_lock(port_t *port)
+int HAMLIB_API par_lock(hamlib_port_t *port)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	if (ioctl(port->fd, PPCLAIM) < 0) {
@@ -255,13 +290,14 @@ int HAMLIB_API par_lock(port_t *port)
 		return -RIG_EIO;
 	}
 	return RIG_OK;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
 #elif defined(WIN32)
 	return RIG_OK;
 #endif
 	return -RIG_ENIMPL;
 }
 
-int HAMLIB_API par_unlock(port_t *port)
+int HAMLIB_API par_unlock(hamlib_port_t *port)
 {
 #ifdef HAVE_LINUX_PPDEV_H
 	if (ioctl(port->fd, PPRELEASE) < 0) {
@@ -269,6 +305,7 @@ int HAMLIB_API par_unlock(port_t *port)
 		return -RIG_EIO;
 	}
 	return RIG_OK;
+#elif defined(HAVE_DEV_PPBUS_PPI_H)
 #elif defined(WIN32)
 	return RIG_OK;
 #endif
@@ -276,77 +313,80 @@ int HAMLIB_API par_unlock(port_t *port)
 }
 
 
-int par_ptt_set(port_t *p, ptt_t pttx)
+int par_ptt_set(hamlib_port_t *p, ptt_t pttx)
 {
-		switch(p->type.ptt) {
-		case RIG_PTT_PARALLEL:
-				{
-					unsigned char reg;
-					int status;
+	switch(p->type.ptt) {
+	case RIG_PTT_PARALLEL:
+		{
+		unsigned char reg;
+		int status;
 
-					status = par_read_data(p, &reg);
-					if (status != RIG_OK)
-						return status;
-					if (pttx == RIG_PTT_ON)
-						reg |=   1 << p->parm.parallel.pin;
-					else
-						reg &= ~(1 << p->parm.parallel.pin);
+		status = par_read_data(p, &reg);
+		if (status != RIG_OK)
+			return status;
+		if (pttx == RIG_PTT_ON)
+			reg |=   1 << p->parm.parallel.pin;
+		else
+			reg &= ~(1 << p->parm.parallel.pin);
 
-					return par_write_data(p, reg);
-				}
-		default:
-				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
-								p->type.ptt);
-				return -RIG_EINVAL;
+		return par_write_data(p, reg);
 		}
-		return RIG_OK;
+	default:
+		rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
+						p->type.ptt);
+		return -RIG_EINVAL;
+	}
+
+	return RIG_OK;
 }
 
 /*
  * assumes pttx not NULL
  */
-int par_ptt_get(port_t *p, ptt_t *pttx)
+int par_ptt_get(hamlib_port_t *p, ptt_t *pttx)
 {
-		switch(p->type.ptt) {
-		case RIG_PTT_PARALLEL:
-				{
-					unsigned char reg;
-					int status;
+	switch(p->type.ptt) {
+	case RIG_PTT_PARALLEL:
+		{
+		unsigned char reg;
+		int status;
 
-					status = par_read_data(p, &reg);
-					*pttx = reg & (1<<p->parm.parallel.pin) ? 
-							RIG_PTT_ON:RIG_PTT_OFF;
-					return status;
-				}
-		default:
-				rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
-								p->type.ptt);
-				return -RIG_ENAVAIL;
+		status = par_read_data(p, &reg);
+		*pttx = reg & (1<<p->parm.parallel.pin) ? 
+					RIG_PTT_ON:RIG_PTT_OFF;
+		return status;
 		}
-		return RIG_OK;
+	default:
+		rig_debug(RIG_DEBUG_ERR,"Unsupported PTT type %d\n", 
+						p->type.ptt);
+		return -RIG_ENAVAIL;
+	}
+
+	return RIG_OK;
 }
 
 /*
  * assumes dcdx not NULL
  */
-int par_dcd_get(port_t *p, dcd_t *dcdx)
+int par_dcd_get(hamlib_port_t *p, dcd_t *dcdx)
 {
-		switch(p->type.dcd) {
-		case RIG_DCD_PARALLEL:
-				{
-					unsigned char reg;
-					int status;
+	switch(p->type.dcd) {
+	case RIG_DCD_PARALLEL:
+		{
+		unsigned char reg;
+		int status;
 
-					status = par_read_data(p, &reg);
-					*dcdx = reg & (1<<p->parm.parallel.pin) ? 
-							RIG_DCD_ON:RIG_DCD_OFF;
-					return status;
-				}
-		default:
-				rig_debug(RIG_DEBUG_ERR,"Unsupported DCD type %d\n", 
-								p->type.dcd);
-				return -RIG_ENAVAIL;
+		status = par_read_data(p, &reg);
+		*dcdx = reg & (1<<p->parm.parallel.pin) ? 
+				RIG_DCD_ON:RIG_DCD_OFF;
+		return status;
 		}
-		return RIG_OK;
+	default:
+		rig_debug(RIG_DEBUG_ERR,"Unsupported DCD type %d\n", 
+						p->type.dcd);
+		return -RIG_ENAVAIL;
+	}
+
+	return RIG_OK;
 }
 
