@@ -3,13 +3,13 @@
  * \ingroup rig
  * \brief CTCSS and DCS interface and tables
  * \author Stephane Fillod
- * \date 2000-2004
+ * \date 2000-2006
  */
 /*
  *  Hamlib Interface - CTCSS and DCS interface and tables
- *  Copyright (c) 2000-2004 by Stephane Fillod
+ *  Copyright (c) 2000-2006 by Stephane Fillod
  *
- *	$Id: tones.c,v 1.7 2004/10/02 10:32:09 fillods Exp $
+ *	$Id: tones.c,v 1.9 2006/02/26 19:29:19 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -101,7 +101,7 @@ int HAMLIB_API rig_set_ctcss_tone(RIG *rig, vfo_t vfo, tone_t tone)
 	if (caps->set_ctcss_tone == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->set_ctcss_tone(rig, vfo, tone);
 
@@ -150,7 +150,7 @@ int HAMLIB_API rig_get_ctcss_tone(RIG *rig, vfo_t vfo, tone_t *tone)
 	if (caps->get_ctcss_tone == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->get_ctcss_tone(rig, vfo, tone);
 
@@ -195,7 +195,7 @@ int HAMLIB_API rig_set_dcs_code(RIG *rig, vfo_t vfo, tone_t code)
 	if (caps->set_dcs_code == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->set_dcs_code(rig, vfo, code);
 
@@ -239,7 +239,7 @@ int HAMLIB_API rig_get_dcs_code(RIG *rig, vfo_t vfo, tone_t *code)
 	if (caps->get_dcs_code == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->get_dcs_code(rig, vfo, code);
 
@@ -289,7 +289,7 @@ int HAMLIB_API rig_set_ctcss_sql(RIG *rig, vfo_t vfo, tone_t tone)
 	if (caps->set_ctcss_sql == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) || 
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) || 
 				vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->set_ctcss_sql(rig, vfo, tone);
 
@@ -338,7 +338,7 @@ int HAMLIB_API rig_get_ctcss_sql(RIG *rig, vfo_t vfo, tone_t *tone)
 	if (caps->get_ctcss_sql == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 			vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->get_ctcss_sql(rig, vfo, tone);
 
@@ -383,7 +383,7 @@ int HAMLIB_API rig_set_dcs_sql(RIG *rig, vfo_t vfo, tone_t code)
 	if (caps->set_dcs_sql == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 			vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->set_dcs_sql(rig, vfo, code);
 
@@ -427,7 +427,7 @@ int HAMLIB_API rig_get_dcs_sql(RIG *rig, vfo_t vfo, tone_t *code)
 	if (caps->get_dcs_sql == NULL)
 		return -RIG_ENAVAIL;
 
-	if ((caps->targetable_vfo&RIG_TARGETABLE_ALL) ||
+	if ((caps->targetable_vfo&RIG_TARGETABLE_PURE) ||
 			vfo == RIG_VFO_CURR || vfo == rig->state.current_vfo)
 		return caps->get_dcs_sql(rig, vfo, code);
 
