@@ -2,7 +2,7 @@
  *  Hamlib TenTenc backend - TT-565 description
  *  Copyright (c) 2004-2005 by Stephane Fillod & Martin Ewing
  *
- *	$Id: orion.c,v 1.9 2005/04/03 20:21:25 fillods Exp $
+ *	$Id: orion.c,v 1.15 2005/04/15 15:30:09 aa6e Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -21,14 +21,27 @@
  */
 
 /* Edits by Martin Ewing AA6E, 23 Mar 2005 --> ?? 
-   Added valid length settings before tentec_transaction calls.
-   Added vfo_curr initialization to VFO A
-   Fixed up VSWR & S-meter, set ATT, set AGC, add rough STR_CAL func.
-   Use local tt565_transaction due to quirky serial interface
-   Variable-length transaction read ok.
-   Calibrated S-meter response with signal generator.
-   Read re-tries implemented.
-*/
+ * Added valid length settings before tentec_transaction calls.
+ * Added vfo_curr initialization to VFO A
+ * Fixed up VSWR & S-meter, set ATT, set AGC, add rough STR_CAL func.
+ * Use local tt565_transaction due to quirky serial interface
+ * Variable-length transaction read ok.
+ * Calibrated S-meter response with signal generator.
+ * Read re-tries implemented.
+ * Added RIG_LEVEL_CWPITCH, RIG_LEVEL_KEYSPD, send_morse()
+ * Added RIG_FUNC_TUNER, RIG_FUNC_LOCK and RIG_FUNC_VOX, fixed MEM_CAP.
+ * Added VFO_OPS
+ * Support LEVEL_VOX, VOXGAIN, ANTIVOX
+ * Support LEVEL_NR as Orion NB setting (firmware bug), FUNC_NB -> NB=0,4
+ * Add get_, set_ant (ignores rx only ant)
+ */
+
+/* Known issues & to-do list:
+ * Memory channels - emulate a more complete memory system?
+ * Send_Morse() - needs to buffer more than 20 chars?
+ * Figure out "granularities".
+ * XCHG or other "fancy" VFO & MEM operations?
+ */
 
 #ifdef HAVE_CONFIG_H
 #include "config.h"
@@ -47,17 +60,18 @@
 #include "idx_builtin.h"
 #include "tentec.h"
 
+#define TT565_BUFSIZE 16
+
 /*
- * Mem caps, to be checked..
+ * Orion's own  memory channel holds a freq, mode, and bandwidth.
+ * May be captured from VFO A or B and applied to VFO A or B.
+ * It cannot directly be read or written from the computer! 
  */
+ 
 #define TT565_MEM_CAP {        \
 	.freq = 1,      \
 	.mode = 1,      \
 	.width = 1,     \
-	.tx_freq = 1,	\
-	.tx_mode = 1,	\
-	.tx_width = 1,	\
-	.split = 1,	\
 }
 
 static int tt565_init(RIG *rig);
@@ -85,26 +99,31 @@ static int tt565_get_xit(RIG * rig, vfo_t vfo, shortfreq_t *xit);
 static int tt565_set_level(RIG * rig, vfo_t vfo, setting_t level, value_t val);
 static int tt565_get_level(RIG * rig, vfo_t vfo, setting_t level, value_t *val);
 static const char* tt565_get_info(RIG *rig);
+static int tt565_send_morse(RIG *rig, vfo_t vfo, const char *msg);
+static int tt565_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status);
+static int tt565_set_func(RIG *rig, vfo_t vfo, setting_t func, int status);
+static int tt565_set_ant(RIG * rig, vfo_t vfo, ant_t ant);
+static int tt565_get_ant(RIG *rig, vfo_t vfo, ant_t *ant);
 
 struct tt565_priv_data {
 	int ch;		/* mem */
 	vfo_t vfo_curr;
 };
 
-
 #define TT565_MODES (RIG_MODE_FM|RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_SSB|\
 			RIG_MODE_RTTY|RIG_MODE_AM)
 #define TT565_RXMODES (TT565_MODES)
 
-#define TT565_FUNCS (RIG_FUNC_LOCK|RIG_FUNC_TUNER)
+#define TT565_FUNCS (RIG_FUNC_LOCK|RIG_FUNC_TUNER|RIG_FUNC_VOX|RIG_FUNC_NB)
 
-#define TT565_LEVELS (RIG_LEVEL_RAWSTR|/*RIG_LEVEL_NB|*/ \
+#define TT565_LEVELS (RIG_LEVEL_RAWSTR| \
+				RIG_LEVEL_CWPITCH| \
 				RIG_LEVEL_SQL|RIG_LEVEL_IF| \
 				RIG_LEVEL_RFPOWER|RIG_LEVEL_KEYSPD| \
 				RIG_LEVEL_RF|RIG_LEVEL_NR| \
-				/*RIG_LEVEL_ANF|*/RIG_LEVEL_MICGAIN| \
+				RIG_LEVEL_MICGAIN| \
 				RIG_LEVEL_AF|RIG_LEVEL_AGC| \
-				RIG_LEVEL_VOXGAIN|RIG_LEVEL_VOX| \
+				RIG_LEVEL_VOXGAIN|RIG_LEVEL_VOX|RIG_LEVEL_ANTIVOX| \
 				RIG_LEVEL_COMP|RIG_LEVEL_PREAMP| \
 				RIG_LEVEL_SWR|RIG_LEVEL_ATT)
 
@@ -138,67 +157,8 @@ struct tt565_priv_data {
                 { 101,  65 }, /* at high end of scale */ \
         } }
 
-#define TT565_EOM "\r"		/* <cr> terminates input string */
-
 #undef TT565_TIME		/* Define to enable time checks */
 
-#ifdef TT565_TIME
-double tt565_timenow()	/* returns current time in secs+microsecs */
-{
-	struct timeval tv;
-	gettimeofday(&tv, NULL);
-	return (double)tv.tv_sec + ((double)tv.tv_usec)/1.0e+6;
-}
-#endif
-
-/*
- * tt565_transaction, adapted from tentec_transaction (tentec.c)
- * read exactly data_len bytes
- * We assume that rig!=NULL, rig->state!= NULL, data!=NULL, data_len!=NULL
- * Otherwise, you'll get a nice seg fault. You've been warned!
- */
-int tt565_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
-{
-        int retval, data_len_init, itry;
-        struct rig_state *rs;
-#ifdef TT565_TIME
- 	double ft1, ft2;	
-#endif
-	/* Capture buffer length for possible read re-try. */
-	data_len_init = (data && data_len) ? *data_len : 0;
-	/* Allow transaction re-tries according to capabilities. */
-	for (itry=1; itry < rig->caps->retry; itry++) {	
-          rs = &rig->state;
-
-          serial_flush(&rs->rigport);
-
-          retval = write_block(&rs->rigport, cmd, cmd_len);
-          if (retval != RIG_OK)
-                        return retval;
-
-          /* no data expected, TODO: flush input? */
-          if (!data || !data_len)
-                        return 0;	/* normal exit if no read */
-#ifdef TT565_TIME
-	  ft1 = tt565_timenow();
-#endif
-	  *data_len = data_len_init;	/* restore orig. buffer length */
-          *data_len = read_string(&rs->rigport, data, *data_len, 
-		TT565_EOM, strlen(TT565_EOM));
-	  if (*data_len > 0) return RIG_OK; /* normal exit if reading */
-#ifdef TT565_TIME
-	  ft2 = tt565_timenow();
-          if (*data_len == -RIG_ETIMEOUT)
-	    rig_debug(RIG_DEBUG_ERR,"Timeout %d: Elapsed = %f secs.\n", itry, ft2-ft1);
-	  else
-	    rig_debug(RIG_DEBUG_ERR,"Other Error #%d, itry=%d: Elapsed = %f secs.\n", 
-		*data_len, itry, ft2-ft1);
-#endif
-	}
-        return RIG_ETIMEOUT;
-}
-
-
 /*
  * tt565 transceiver capabilities.
  *
@@ -211,7 +171,7 @@ const struct rig_caps tt565_caps = {
 .mfg_name =  "Ten-Tec",
 .version =  "0.3",
 .copyright =  "LGPL",
-.status =  RIG_STATUS_ALPHA,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_RIG,
 .dcd_type =  RIG_DCD_NONE,
@@ -223,7 +183,7 @@ const struct rig_caps tt565_caps = {
 .serial_parity =  RIG_PARITY_NONE,
 .serial_handshake =  RIG_HANDSHAKE_HARDWARE,
 .write_delay =  0,
-.post_write_delay =  0,
+.post_write_delay =  10,	/* Needed for CW send + ?? */
 .timeout =  400,
 .retry =  3,
 
@@ -238,11 +198,12 @@ const struct rig_caps tt565_caps = {
 .parm_gran =  {},
 .ctcss_list =  NULL,
 .dcs_list =  NULL,
-.preamp =   { 20, RIG_DBLST_END },	/* TBC */
+.preamp =   { 20, RIG_DBLST_END },
 .attenuator =   { 6, 12, 18, RIG_DBLST_END },
 .max_rit =  kHz(8),
 .max_xit =  kHz(8),
 .max_ifshift =  kHz(8),
+.vfo_ops = TT565_VFO_OPS,
 .targetable_vfo =  RIG_TARGETABLE_ALL,
 .transceive =  RIG_TRN_OFF,
 .bank_qty =   0,
@@ -324,6 +285,11 @@ const struct rig_caps tt565_caps = {
 .get_xit =  tt565_get_xit,
 .reset =  tt565_reset,
 .get_info =  tt565_get_info,
+.send_morse = tt565_send_morse,
+.get_func = tt565_get_func,
+.set_func = tt565_set_func,
+.get_ant = tt565_get_ant,
+.set_ant = tt565_set_ant,
 
 .str_cal = TT565_STR_CAL,
 };
@@ -342,16 +308,66 @@ const struct rig_caps tt565_caps = {
 #define TT565_FM  '5'
 #define TT565_RTTY '6'
 
-/*************************************************************************************
- *
- * Specs from http://www.rfsquared.com, Rev 1, firmware 1.340
- *
- * 	[sg]et_func
- * 	[sg]et_ant
- *
- * 	XCHG
- */
+#ifdef TT565_TIME
+double tt565_timenow()	/* returns current time in secs+microsecs */
+{
+	struct timeval tv;
+	gettimeofday(&tv, NULL);
+	return (double)tv.tv_sec + ((double)tv.tv_usec)/1.0e+6;
+}
+#endif
 
+/*
+ * tt565_transaction, adapted from tentec_transaction (tentec.c)
+ * read variable number of bytes, up to buffer size, if data & data_len != NULL.
+ * We assume that rig!=NULL, rig->state!= NULL
+ * Otherwise, you'll get a nice seg fault. You've been warned!
+ */
+int tt565_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
+{
+        int retval, data_len_init, itry;
+        struct rig_state *rs;
+#ifdef TT565_TIME
+ 	double ft1, ft2;	
+#endif
+	/* Capture buffer length for possible read re-try. */
+	data_len_init = (data && data_len) ? *data_len : 0;
+	/* Allow transaction re-tries according to capabilities. */
+	for (itry=1; itry < rig->caps->retry; itry++) {	
+          rs = &rig->state;
+
+          serial_flush(&rs->rigport);
+
+          retval = write_block(&rs->rigport, cmd, cmd_len);
+          if (retval != RIG_OK)
+                        return retval;
+
+          /* no data expected, TODO: flush input? */
+          if (!data || !data_len)
+                        return 0;	/* normal exit if no read */
+#ifdef TT565_TIME
+	  ft1 = tt565_timenow();
+#endif
+	  *data_len = data_len_init;	/* restore orig. buffer length */
+          *data_len = read_string(&rs->rigport, data, *data_len, 
+		EOM, strlen(EOM));
+	  if (*data_len > 0) return RIG_OK; /* normal exit if reading */
+#ifdef TT565_TIME
+	  ft2 = tt565_timenow();
+          if (*data_len == -RIG_ETIMEOUT)
+	    rig_debug(RIG_DEBUG_ERR,"Timeout %d: Elapsed = %f secs.\n", itry, ft2-ft1);
+	  else
+	    rig_debug(RIG_DEBUG_ERR,"Other Error #%d, itry=%d: Elapsed = %f secs.\n", 
+		*data_len, itry, ft2-ft1);
+#endif
+	}
+        return -RIG_ETIMEOUT;
+}
+
+/*************************************************************************************
+ * Specs from http://www.rfsquared.com, Rev 1, firmware 1.340
+ * supplemented and corrected with real hardware, firmware 1.372
+ */
 
 /*
  * tt565_init:
@@ -440,7 +456,7 @@ static char which_vfo(const RIG *rig, vfo_t vfo)
 int tt565_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
 	int cmd_len, retval;
-	unsigned char cmdbuf[16];
+	unsigned char cmdbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf (cmdbuf, "*%cF%"PRIll EOM, 
 			which_vfo(rig, vfo),
@@ -458,7 +474,7 @@ int tt565_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 int tt565_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "?%cF" EOM,
 				which_vfo(rig, vfo));
@@ -487,12 +503,13 @@ int tt565_set_vfo(RIG *rig, vfo_t vfo)
 {
 	struct tt565_priv_data *priv = (struct tt565_priv_data *)rig->state.priv;
 	int vfo_len;
-	unsigned char vfobuf[16];
+	unsigned char vfobuf[TT565_BUFSIZE];
 
 	if (vfo == RIG_VFO_CURR)
 		return RIG_OK;
 
 	if (vfo == RIG_VFO_MAIN || vfo == RIG_VFO_SUB) {
+		/* Select Sub or Main RX */
 		vfo_len = sprintf (vfobuf, "*K%c" EOM, 
 				vfo == RIG_VFO_SUB ? 'S' : 'M');
 
@@ -524,7 +541,7 @@ int tt565_get_vfo(RIG *rig, vfo_t *vfo)
 int tt565_set_split_vfo(RIG *rig, vfo_t vfo, split_t split, vfo_t tx_vfo)
 {
 	int cmd_len, retval;
-	unsigned char cmdbuf[16];
+	unsigned char cmdbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf (cmdbuf, "*KV%c%c%c" EOM, 
 			which_vfo(rig, vfo),
@@ -553,7 +570,7 @@ static vfo_t tt2vfo(char c)
 int tt565_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 	char ttreceiver;
 
 	cmd_len = sprintf(cmdbuf, "?KV" EOM);
@@ -578,7 +595,6 @@ int tt565_get_split_vfo(RIG *rig, vfo_t vfo, split_t *split, vfo_t *tx_vfo)
 	return RIG_OK;
 }
 
-
 /*
  * tt565_set_mode
  * Assumes rig!=NULL
@@ -588,7 +604,7 @@ int tt565_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 	struct rig_state *rs = &rig->state;
 	unsigned char ttmode, ttreceiver;
 	int mdbuf_len, retval;
-	unsigned char mdbuf[32];
+	unsigned char mdbuf[TT565_BUFSIZE];
 
 	switch (mode) {
 	case RIG_MODE_USB:	ttmode = TT565_USB; break;
@@ -621,8 +637,6 @@ int tt565_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 	return retval;
 }
 
-
-
 /*
  * tt565_get_mode
  * Assumes rig!=NULL, mode!=NULL
@@ -630,7 +644,7 @@ int tt565_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
 int tt565_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 	char ttmode, ttreceiver;
 
 	ttreceiver = which_receiver(rig, vfo);
@@ -691,7 +705,7 @@ int tt565_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 int tt565_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 {
 	int cmd_len, retval;
-	unsigned char cmdbuf[16];
+	unsigned char cmdbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "*R%cI%d" EOM,
 				which_receiver(rig, vfo),
@@ -705,7 +719,7 @@ int tt565_set_ts(RIG *rig, vfo_t vfo, shortfreq_t ts)
 int tt565_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "?R%cI" EOM,
 				which_receiver(rig, vfo));
@@ -734,7 +748,7 @@ int tt565_get_ts(RIG *rig, vfo_t vfo, shortfreq_t *ts)
 int tt565_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 {
 	int cmd_len, retval;
-	unsigned char cmdbuf[16];
+	unsigned char cmdbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "*R%cR%d" EOM,
 				which_receiver(rig, vfo),
@@ -748,7 +762,7 @@ int tt565_set_rit(RIG *rig, vfo_t vfo, shortfreq_t rit)
 int tt565_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "?R%cR" EOM,
 				which_receiver(rig, vfo));
@@ -778,7 +792,7 @@ int tt565_get_rit(RIG *rig, vfo_t vfo, shortfreq_t *rit)
 int tt565_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 {
 	int cmd_len, retval;
-	unsigned char cmdbuf[16];
+	unsigned char cmdbuf[TT565_BUFSIZE];
 
 	/* Sub receiver does not contain an XIT setting */
 
@@ -794,7 +808,7 @@ int tt565_set_xit(RIG *rig, vfo_t vfo, shortfreq_t xit)
 int tt565_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 {
 	int cmd_len, resp_len, retval;
-	unsigned char cmdbuf[16], respbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
 
 	cmd_len = sprintf(cmdbuf, "?R%cX" EOM,
 				'M');
@@ -816,8 +830,6 @@ int tt565_get_xit(RIG *rig, vfo_t vfo, shortfreq_t *xit)
 	return RIG_OK;
 }
 
-
-
 /*
  * tt565_set_ptt
  * Assumes rig!=NULL
@@ -833,7 +845,7 @@ int tt565_set_ptt(RIG *rig, vfo_t vfo, ptt_t ptt)
 int tt565_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 {
 	int resp_len, retval;
-	unsigned char respbuf[32];
+	unsigned char respbuf[TT565_BUFSIZE];
 
 	resp_len = sizeof(respbuf);	
 	retval = tt565_transaction (rig, "?S" EOM, 3, respbuf, &resp_len);
@@ -858,7 +870,7 @@ int tt565_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 int tt565_reset(RIG *rig, reset_t reset)
 {
 	int retval, reset_len;
-	char reset_buf[32];
+	char reset_buf[TT565_BUFSIZE];
 
 	reset_len = sizeof(reset_buf);	
 	retval = tt565_transaction (rig, "X" EOM, 2, reset_buf, &reset_len);
@@ -878,8 +890,6 @@ int tt565_reset(RIG *rig, reset_t reset)
 /*
  * tt565_get_info
  * Assumes rig!=NULL
- *
- * FIXME: what is the Orion command to get Firmware version?
  */
 const char *tt565_get_info(RIG *rig)
 {
@@ -911,7 +921,7 @@ const char *tt565_get_info(RIG *rig)
 int tt565_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 {
 	int retval, cmd_len=0, ii;
-	unsigned char cmdbuf[16], cc;
+	unsigned char cmdbuf[TT565_BUFSIZE], cc;
 
 	switch (level) {
 	case RIG_LEVEL_RFPOWER:
@@ -988,6 +998,51 @@ int tt565_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 				(int)(val.f*9));
 		break;
 
+	case RIG_LEVEL_CWPITCH:
+		/* "CWPITCH" is the "Tone" button on the Orion.
+		   Manual menu adjustment works down to 100 Hz, but not via
+		   computer.
+		*/
+		if (val.i > 1200) val.i = 1200;
+		else if (val.i < 300) val.i = 300;	/* Range 300-1200 Hz works */
+							/* Limits should be in caps? */
+		cmd_len = sprintf(cmdbuf, "*CT%d" EOM,
+				val.i);
+		break;
+
+	case RIG_LEVEL_KEYSPD:
+		/* Keyer speed setting does not imply Keyer = "on".  That is a
+		   command which should be a hamlib function, but is not.
+		   Keyer speed determines the rate of computer sent CW also.
+		*/
+		if (val.i > 60) val.i = 60;
+		else if (val.i < 10) val.i = 10;	/* Range 10-60 wpm */	
+		cmd_len = sprintf(cmdbuf, "*CS%d" EOM,
+				val.i);
+		break;
+
+	case RIG_LEVEL_NR:
+		/* For some reason NB setting is supported in 1.372, but
+		   NR, NOTCH, and AN are not. 
+		   FOR NOW -- RIG_LEVEL_NR controls the Orion NB setting 
+		*/
+		cmd_len = sprintf(cmdbuf, "*R%cNB%d" EOM,
+				which_receiver(rig, vfo),
+				(int)(val.f*9));
+		break;
+
+	case RIG_LEVEL_VOX:	/* =VOXDELAY, tenths of seconds */
+		cmd_len = sprintf(cmdbuf, "*TH%4.2f" EOM, 0.1*val.f);
+		break;
+
+	case RIG_LEVEL_VOXGAIN:	/* Float, 0.0 - 1.0 */
+		cmd_len = sprintf(cmdbuf, "*TG%d" EOM, (int)(100.0*val.f));
+		break;
+
+	case RIG_LEVEL_ANTIVOX:	/* Float, 0.0 - 1.0 */
+		cmd_len = sprintf(cmdbuf, "*TA%d" EOM, (int)(100.0*val.f));
+		break;
+
 	default:
 		rig_debug(RIG_DEBUG_ERR,"%s: unsupported level %d\n",
 				__FUNCTION__, level);
@@ -1007,12 +1062,9 @@ int tt565_set_level(RIG *rig, vfo_t vfo, setting_t level, value_t val)
 int tt565_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 {
 	int retval, cmd_len, lvl_len;
-	unsigned char cmdbuf[16],lvlbuf[32];
+	unsigned char cmdbuf[TT565_BUFSIZE],lvlbuf[TT565_BUFSIZE];
 
-
-	/* Optimize:
-	 *   sort the switch cases with the most frequent first
-	 */
+	/* Optimize: sort the switch cases with the most frequent first */
 	switch (level) {
 	case RIG_LEVEL_SWR:
 		lvl_len = sizeof(lvlbuf);	
@@ -1244,6 +1296,90 @@ int tt565_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 		val->f = atof(lvlbuf+3)/9.0;
 		break;
 
+	case RIG_LEVEL_CWPITCH:
+		lvl_len = sizeof(lvlbuf);
+		retval = tt565_transaction (rig, "?CT" EOM, 4, lvlbuf, &lvl_len);
+		if (retval != RIG_OK)
+			return retval;
+
+		if (lvlbuf[1] != 'C' || lvlbuf[2] != 'T' || lvl_len < 4) {
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+					__FUNCTION__, lvlbuf);
+			return -RIG_EPROTO;
+		}
+		val->i = atoi(lvlbuf+3);
+		break;
+
+	case RIG_LEVEL_KEYSPD:
+		lvl_len = sizeof(lvlbuf);
+		retval = tt565_transaction (rig, "?CS" EOM, 4, lvlbuf, &lvl_len);
+		if (retval != RIG_OK)
+			return retval;
+
+		if (lvlbuf[1] != 'C' || lvlbuf[2] != 'S' || lvl_len < 4) {
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+					__FUNCTION__, lvlbuf);
+			return -RIG_EPROTO;
+		}
+		val->i = atoi(lvlbuf+3);
+		break;
+
+	case RIG_LEVEL_NR:
+		/* RIG_LEVEL_NR controls Orion NB setting - TEMP */
+		cmd_len = sprintf(cmdbuf, "?R%cNB" EOM,
+                                which_receiver(rig, vfo));
+	
+		lvl_len = sizeof(lvlbuf);
+		retval = tt565_transaction (rig, cmdbuf, cmd_len, lvlbuf, &lvl_len);
+		if (retval != RIG_OK)
+			return retval;
+		if (lvlbuf[1] != 'R' || lvlbuf[3] != 'N' || lvlbuf[4] != 'B' ||
+			lvl_len < 6 ) {
+				rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+					__FUNCTION__, lvlbuf);
+				return -RIG_EPROTO;
+			}
+		val->f = atof(lvlbuf+5)/9.0;	/* Note 0-9 -> 0.0 - 1.0 */
+		break;
+
+	case RIG_LEVEL_VOX:	/* =VOXDELAY, tenths of secs. */
+		lvl_len = sizeof(lvlbuf);
+		retval = tt565_transaction (rig, "?TH" EOM, 4, lvlbuf, &lvl_len);
+		if (retval != RIG_OK)
+			return retval;
+		if (lvlbuf[1] != 'T' || lvlbuf[2] != 'H' || lvl_len < 4) {
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+				__FUNCTION__, lvlbuf);
+			return -RIG_EPROTO;
+		}
+		val->f = 10.0*atof(lvlbuf+3);
+		break;
+
+	case RIG_LEVEL_VOXGAIN:	/* Float, 0.0 - 1.0 */
+		lvl_len = sizeof(lvlbuf);
+		retval = tt565_transaction (rig, "?TG" EOM, 4, lvlbuf, &lvl_len);
+		if (retval != RIG_OK)
+			return retval;
+		if (lvlbuf[1] != 'T' || lvlbuf[2] != 'G' || lvl_len < 4) {
+			rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+				__FUNCTION__, lvlbuf);
+			return -RIG_EPROTO;
+		}
+		val->f = 0.01 * atof(lvlbuf+3);
+	break;
+
+	case RIG_LEVEL_ANTIVOX:	/* Float, 0.0 - 1.0 */
+                lvl_len = sizeof(lvlbuf);
+                retval = tt565_transaction (rig, "?TA" EOM, 4, lvlbuf, &lvl_len);
+                if (retval != RIG_OK)
+                        return retval;
+                if (lvlbuf[1] != 'T' || lvlbuf[2] != 'A' || lvl_len < 4) {
+                        rig_debug(RIG_DEBUG_ERR,"%s: unexpected answer '%s'\n",
+                                __FUNCTION__, lvlbuf);
+                        return -RIG_EPROTO;
+                }
+                val->f = 0.01 * atof(lvlbuf+3);
+	break;
 
 	default:
 		rig_debug(RIG_DEBUG_ERR,"%s: unsupported level %d\n", 
@@ -1258,8 +1394,7 @@ int tt565_set_mem(RIG * rig, vfo_t vfo, int ch)
 {
 	struct tt565_priv_data *priv = (struct tt565_priv_data *)rig->state.priv;
 
-	priv->ch = ch;
-
+	priv->ch = ch;	/* See RIG_OP_TO/FROM_VFO */
 	return RIG_OK;
 }
 
@@ -1268,14 +1403,13 @@ int tt565_get_mem(RIG * rig, vfo_t vfo, int *ch)
 	struct tt565_priv_data *priv = (struct tt565_priv_data *)rig->state.priv;
 
 	*ch = priv->ch;
-
 	return RIG_OK;
 }
 
 int tt565_vfo_op(RIG * rig, vfo_t vfo, vfo_op_t op)
 {
 	struct tt565_priv_data *priv = (struct tt565_priv_data *)rig->state.priv;
-	char cmdbuf[16];
+	char cmdbuf[TT565_BUFSIZE];
 	int retval;
 	int cmd_len;
 
@@ -1307,7 +1441,246 @@ int tt565_vfo_op(RIG * rig, vfo_t vfo, vfo_op_t op)
 	}
 
 	retval = tt565_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
-
 	return retval;
 }
+
+int tt565_send_morse(RIG *rig, vfo_t vfo, const char *msg)
+{
+	int msg_len, retval, ic, cmdl;
+	char morsecmd[8];
+	static int keyer_set = 0;	/*Shouldn't be here!*/
+
+/* Orion keyer must be on for morse, but we do not have a "keyer on" function in 
+ * hamlib (yet).  So force keyer on here. 
+ */
+	if (!keyer_set) {
+	    retval = tt565_transaction(rig, "*CK1" EOM, 5, NULL, NULL);
+	    if (retval != RIG_OK)
+		return retval;
+	    keyer_set = 1;
+	    usleep(100000);	/* 100 msec - guess */
+	}
+	msg_len = strlen(msg);
+	if (msg_len > 20) msg_len = 20;	/* sanity limit 20 chars */
+
+/* Orion can queue up to about 20 characters.  
+ * We could batch a longer message into 20 char chunks, but there is no
+ * simple way to tell if message has completed.  We could calculate a
+ * duration based on keyer speed and the text that was sent, but
+ * what we really need is a handshake for "message complete".
+ * Without it, you can't easily use the Orion as a code practice machine.
+ * For now, we let the user do the batching.
+ * Note that rig panel is locked up for duration of message. 
+ */
+	for (ic = 0; ic < msg_len; ic++) {
+		cmdl = sprintf(morsecmd,"/%c" EOM, msg[ic]);
+		retval = tt565_transaction(rig,morsecmd,cmdl,NULL,NULL);
+		if (retval != RIG_OK)
+			return retval;
+	}
+	return RIG_OK;
+}
+
+int tt565_set_func(RIG *rig, vfo_t vfo, setting_t func, int status)
+{
+	unsigned char fcmdbuf[TT565_BUFSIZE];
+	int retval, fcmdlen;
+
+	if (vfo != RIG_VFO_CURR)
+		return -RIG_EINVAL;
+
+	switch (func) {
+	case RIG_FUNC_TUNER:
+		fcmdlen = sprintf(fcmdbuf,"*TT%c" EOM, !status ? 0:1);
+		break;
+
+	case RIG_FUNC_VOX:
+		fcmdlen = sprintf(fcmdbuf,"*TV%c" EOM, !status ? 0:1);
+		break;
+
+	case RIG_FUNC_LOCK:
+		fcmdlen = sprintf(fcmdbuf,"*%c%c" EOM,
+			which_vfo(rig, vfo),
+			!status ? 'U' : 'L' );
+		break;
+
+	case RIG_FUNC_NB:
+		/* NB "on" sets Orion NB=4; "off" -> NB=0.  See also
+		RIG_LEVEL_NR which maps to NB setting due to firmware
+		limitation.
+		*/
+		fcmdlen = sprintf(fcmdbuf,"*R%cNB%c" EOM,
+			which_receiver(rig, vfo),
+			!status ? '0' : '4' );
+		break;
+
+	default:
+                rig_debug(RIG_DEBUG_ERR,"Unsupported set_func %#x", func);
+                return -RIG_EINVAL;
+	}	
+        retval = tt565_transaction(rig, fcmdbuf, fcmdlen, NULL, NULL);
+
+        if (retval != RIG_OK)
+                return retval;
+	return RIG_OK;
+}
+
+int tt565_get_func(RIG *rig, vfo_t vfo, setting_t func, int *status)
+{
+	unsigned char fcmdbuf[TT565_BUFSIZE], frespbuf[TT565_BUFSIZE];
+	int retval, fcmdlen, fresplen;
+
+	if (vfo != RIG_VFO_CURR)
+		return -RIG_EINVAL;
+
+	switch (func) {
+	case RIG_FUNC_TUNER:
+		fcmdlen = sprintf(fcmdbuf, "?TT" EOM);
+		break;
+
+	case RIG_FUNC_VOX:
+		fcmdlen = sprintf(fcmdbuf, "?TV" EOM);
+		break;
+
+	case RIG_FUNC_LOCK:
+		fcmdlen = sprintf(fcmdbuf, "?%cU" EOM, 
+			which_vfo(rig, vfo) );
+		/* needs special treatment */
+		fresplen = sizeof(frespbuf);
+		retval = tt565_transaction(rig, fcmdbuf, fcmdlen, 
+			frespbuf, &fresplen);
+		if (retval != RIG_OK)
+			return retval;
+		/* response is @AL @AU or @BL @BU */
+		*status = frespbuf[ 2 ] == 'L' ? 1:0;
+		return RIG_OK;
+
+	case RIG_FUNC_NB:
+		/* Note NB should be a LEVEL for Orion. It is also
+		   available through LEVEL_NR
+		*/
+		fcmdlen = sprintf(fcmdbuf, "?R%cNB" EOM,
+			which_receiver(rig, vfo) );
+		/* needs special treatment */
+		fresplen = sizeof(frespbuf);
+		retval = tt565_transaction(rig, fcmdbuf, fcmdlen, 
+			frespbuf, &fresplen);
+		if (retval != RIG_OK)
+			return retval;
+		/* response is @RxNBn, n=0--9. Return 0 iff receive NB=0 */
+		*status = frespbuf[ 5 ] == '0' ? 0:1;
+		return RIG_OK;
+
+	default:
+		rig_debug(RIG_DEBUG_ERR,"Unsupported get_func %#x", func);
+		return -RIG_EINVAL;
+	}
+	fresplen = sizeof(frespbuf);
+	retval = tt565_transaction(rig, fcmdbuf, fcmdlen, frespbuf, &fresplen);
+	if (retval != RIG_OK)
+		return retval;
+	*status = frespbuf[ 3 ] == '1' ? 1:0;
+	return RIG_OK;
+}
+/* Antenna selection for Orion
+ * We support Ant_1 and Ant_2 for M and S receivers.
+ * Note that Rx-only antenna (Ant_3?) is not supported at this time.
+ * Orion command assigns MSBN to each ant, but hamlib wants to assign ant to rx/tx!
+ * The efficient way would be to keep current config in rig priv area, but we will
+ * ask the rig what its state is each time...
+ */
+int tt565_set_ant(RIG * rig, vfo_t vfo, ant_t ant)
+{
+                unsigned char respbuf[TT565_BUFSIZE];
+                int resp_len, retval;
+		ant_t main_ant, sub_ant;
+
+		/* First, find out what antenna config is now. */
+		resp_len = sizeof(respbuf);
+		retval = tt565_transaction (rig, "?KA" EOM, 4, respbuf, &resp_len);
+                if (retval != RIG_OK)
+                                return retval;
+		if (resp_len != 7 || respbuf[1] != 'K' || respbuf[2] != 'A') {
+			rig_debug(RIG_DEBUG_ERR,"%s; tt565_set_ant: ?KA NG %s\n", 
+				__FUNCTION__, respbuf);
+			return -RIG_EPROTO;
+		}
+
+		/* respbuf="@KAxxx"
+		 * x='M'|'S'|'B'|'N'=main/sub/both/none for ants 1,2,3. 
+		 * but hardware will not permit all combinations!
+		 * respbuf [3,4] can be MS, SM, BN, NB
+		 * decode to rx-centric view 
+		 */
+		if (respbuf[3] == 'M' || respbuf[3] == 'B') main_ant = RIG_ANT_1;
+			else main_ant = RIG_ANT_2;
+		if (respbuf[3] == 'S' || respbuf[3] == 'B') sub_ant = RIG_ANT_1;
+			else sub_ant = RIG_ANT_2;
+		switch (which_receiver(rig,vfo)) {
+			case 'M':
+				main_ant = ant;
+				break;
+			case 'S':
+				sub_ant = ant;
+				break;
+			default: {
+				/* no change? */
+			}
+		}
+		/* re-encode ant. settings into command */
+		if (main_ant == RIG_ANT_1) {
+			if (sub_ant == RIG_ANT_1) {
+					respbuf[3] = 'B';
+					respbuf[4] = 'N';
+			}
+				else {
+					respbuf[3] = 'M';
+					respbuf[4] = 'S';
+				}
+		}
+		else if (sub_ant == RIG_ANT_2) {
+					respbuf[3] = 'N';
+					respbuf[4] = 'B';
+			}
+				else {
+					respbuf[3] = 'S';
+					respbuf[4] = 'M';
+				}
+		respbuf[0] = '*';	/* respbuf becomes a store command */
+		respbuf[5] = 'N';	/* Force no rx on Ant 3 */
+		respbuf[6] = EOM[0];
+		respbuf[7] = 0;
+                retval = tt565_transaction (rig, respbuf, 7, NULL, NULL); 
+                if (retval != RIG_OK)
+                                return retval;
+                return RIG_OK;
+}
+
+int tt565_get_ant(RIG *rig, vfo_t vfo, ant_t *ant)
+{
+                unsigned char respbuf[TT565_BUFSIZE];
+                int resp_len, retval;
+
+		resp_len = sizeof(respbuf);
+                retval = tt565_transaction(rig, "?KA" EOM, 4, respbuf, &resp_len); 
+                if (retval != RIG_OK)
+                                return retval;
+		if (respbuf[1] != 'K' || respbuf[2] != 'A' || resp_len != 7) {
+			rig_debug(RIG_DEBUG_ERR,"%s; tt565_get_ant: NG %s\n", 
+				__FUNCTION__, respbuf);
+			return -RIG_EPROTO;
+		}
+		/* Look for first occurrence of M or S in ant 1, 2, 3 characters */
+		if (respbuf[3] == which_receiver(rig,vfo) || respbuf[3] == 'B' ) {
+			*ant = RIG_ANT_1;
+			return RIG_OK;
+		}
+		if (respbuf[4] == which_receiver(rig,vfo) || respbuf[4] == 'B' ) {
+			*ant = RIG_ANT_2;
+			return RIG_OK;
+		} 
+
+                *ant = RIG_ANT_NONE;	/* ignore possible RIG_ANT_3 = rx only ant */
+                return RIG_OK;
+}
 /* End of orion.c */
