@@ -1,8 +1,8 @@
 /*
  *  Hamlib Uniden backend - main file
- *  Copyright (c) 2001-2003 by Stephane Fillod
+ *  Copyright (c) 2001-2005 by Stephane Fillod
  *
- *	$Id: uniden.c,v 1.6 2003/04/16 22:30:43 fillods Exp $
+ *	$Id: uniden.c,v 1.11 2005/04/10 21:47:14 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,6 +24,7 @@
 #include "config.h"
 #endif
 
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>  /* String function definitions */
 #include <unistd.h>  /* UNIX standard function definitions */
@@ -55,7 +56,7 @@
  * We assume that rig!=NULL, rig->state!= NULL, data!=NULL, data_len!=NULL
  * Otherwise, you'll get a nice seg fault. You've been warned!
  */
-int uniden_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
+static int uniden_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *data_len)
 {
 	int retval;
 	struct rig_state *rs;
@@ -71,9 +72,14 @@ int uniden_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *
 
 	/* no data expected, TODO: flush input? */
 	if (!data || !data_len)
-			return 0;
+		return 0;
 
-	*data_len = read_string(&rs->rigport, data, BUFSZ, "\x0a", 1);
+	retval = read_string(&rs->rigport, data, BUFSZ, "\x0a", 1);
+	if (retval == -RIG_ETIMEOUT)
+		retval = 0;
+	if (retval < 0)
+		return retval;
+	*data_len = retval;
 
 	return RIG_OK;
 }
@@ -92,7 +98,7 @@ int uniden_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 				return -RIG_EINVAL;
 
 		/* exactly 8 digits */
-		freq_len = sprintf(freqbuf, "RF%08Ld" EOM, freq);
+		freq_len = sprintf(freqbuf, "RF%08"PRIll EOM, (long long)freq);
 
 		return uniden_transaction (rig, freqbuf, freq_len, NULL, NULL);
 }
@@ -119,6 +125,7 @@ DECLARE_INITRIG_BACKEND(uniden)
 		rig_debug(RIG_DEBUG_VERBOSE, "uniden: _init called\n");
 
 		rig_register(&bc895_caps);
+		rig_register(&bc245_caps);
 
 		return RIG_OK;
 }
