@@ -6,7 +6,7 @@
  * via serial interface to an FT-847 using the "CAT" interface.
  *
  *
- * $Id: ft847.c,v 1.29 2004/11/17 22:02:03 fillods Exp $  
+ * $Id: ft847.c,v 1.30 2005/01/25 00:21:58 fillods Exp $  
  *
  *
  *
@@ -453,7 +453,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
 
   rig_s = &rig->state;
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq = %lli Hz \n", freq);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq = %"PRIfreq" Hz \n", freq);
 
 
   /* 
@@ -480,7 +480,7 @@ int ft847_set_freq(RIG *rig, vfo_t vfo, freq_t freq) {
   to_bcd_be(p->p_cmd,freq/10,8);	/* store bcd format in in p_cmd */
 				/* TODO -- fix 10Hz resolution -- FS */
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq after conversion = %lli Hz \n", from_bcd_be(p->p_cmd,8)* 10 );
+  rig_debug(RIG_DEBUG_VERBOSE,"ft847: requested freq after conversion = %"PRIll" Hz \n", from_bcd_be(p->p_cmd,8)* 10 );
 
   cmd = p->p_cmd; /* get native sequence */
   write_block(&rig_s->rigport, cmd, YAESU_CMD_LENGTH);
