@@ -13,7 +13,7 @@
  * The starting point for this code was Frank's ft847 implementation.
  *
  *
- *    $Id: ft857.c,v 1.6 2004/09/26 08:35:05 fillods Exp $  
+ *    $Id: ft857.c,v 1.8 2005/04/10 21:49:38 fillods Exp $  
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -391,7 +391,7 @@ static int ft857_get_status(RIG *rig, int status)
     break;
   default:
     rig_debug(RIG_DEBUG_ERR, "ft857_get_status: Internal error!\n");
-    return RIG_EINTERNAL;
+    return -RIG_EINTERNAL;
   }
 
   serial_flush(&rig->state.rigport);
@@ -633,7 +633,7 @@ int ft857_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
   if (vfo != RIG_VFO_CURR)
     return -RIG_ENTARGET;
 
-  rig_debug(RIG_DEBUG_VERBOSE,"ft857: requested freq = %lli Hz\n", freq);
+  rig_debug(RIG_DEBUG_VERBOSE,"ft857: requested freq = %"PRIfreq" Hz\n", freq);
 
   /* fill in the frequency */
   to_bcd_be(data, (freq + 5) / 10, 8);
